  
"
    __________________________________________________
    @Alejandro F. Reimondo 09/02/03
    This proyect sources are property of : Alejandro F. Reimondo
    Its use is NOT legal without written rights.
    Amenabar 3476 Piso: 2 Dto: B.
    Ciudad de Buenos Aires.
    Argentina.
    Tel: (54)-1-703-5586.
    __________________________________________________
"
    (File pathName: 'Particle.Pol') fileIn;close.


	ARObject
		checkBound: 'vreg31w.sll'; "Interface to the Registration Database."
		checkBound: 'voleb31w.sll'; "Support for the basic Component Object Model (COM) used in OLE applications."
		checkBound: 'volec31w.sll'; "Support for building OLE container applications."
		checkBound: 'volea31w.sll'; "Support for OLE automation."
		"checkBound: 'vlstc31.sll'; UNUSED? - Dialogs for making choice from lists in various presentations."
		checkBound: 'volex31w.sll'; "Support for using OLE controls (OCXs) and building OLE control container applications"
		"checkBound: 'volsx31w.sll'; UNUSED - OCX Samples"
		yourself.
!

Object subclass: #ParticleObject
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''    !

ParticleObject subclass: #ParticleAction
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleAction) comment: '
	ParticleActions are the core of the Particle System.
	They directly manipulate particles in particle groups.
	They perform effects such as gravity, explosions, bouncing, etc. to all particles in the CURRENT particle group.
	Typically one creates and initializes one or more particle groups, then at run time it calls particle actions to animate the particles and finally draw the group of particles onto the screen.

	The Particle System uses a discrete time approximation to all actions.
	See pTimeStep for information on improving simulation quality by varying the time step, dt.
	' !

ParticleAction subclass: #CompositeParticleAction
  instanceVariableNames: 
    ' elements '
  classVariableNames: ''
  poolDictionaries: ''   !

CompositeParticleAction subclass: #ParticleActionList
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleActionList) comment: '
	Action lists are blocks of actions that are applied together to a particle group.
	They are conceptually similar to scripts or procedures.
	An action list abstracts the specifics of a particular effect and allows complex effects to be treated as primitives like actions.
	*Warning* an action list cannot be an action within another action list.
	Action lists also allow effects to be simulated much more efficiently on certain parallel machines, such as PixelFlow.
	Also, action lists can be optimized by a particular implementation of the Particle System.
	For example, common sequences of actions can be detected and replaced by efficient code that handles all those actions in one pass.
	'    !

ParticleAction subclass: #SimpleParticleAction
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''  !

SimpleParticleAction subclass: #DomainParticleAction
  instanceVariableNames: 
    ' domain '
  classVariableNames: ''
  poolDictionaries: ''  !

DomainParticleAction subclass: #DomainAvoid
  instanceVariableNames: 
    ' magnitude epsilon lookAhead '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #DomainAvoid) comment: '
	Steer particles away from a domain of space.
	Particles are tested to see whether they will pass from being outside the specified domain to being inside it within lookAhead time units from now if the next ParticleMove action were to occur now.
	magnitude tells how drastically the particle velocities are modified to avoid the obstacle at each time step.
	As with most acceleration actions, the amount of acceleration falls off inversely with r2.
	But when r is small, the acceleration would be infinite, so epsilon is always added to r.
	The specific direction and amount of turn is dependent on the kind of domain being avoided.
	NOTES
		At present the only domains for which DomainAvoid is implemented are PDSphere, PDRectangle,
		 PDTriangle, PDDisc and PDPlane.
	'   !

DomainParticleAction subclass: #DomainBounce
  instanceVariableNames: 
    ' friction resilience cutoff '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #DomainBounce) comment: '
	Bounce particles off a domain of space.
	Particles are tested to see whether they will pass from being outside the specified domain to being inside it if the next Move action were to occur now.
	If they would pass through the surface of the domain, they are instead bounced off it.
	That is, their velocity vector is decomposed into components normal to the surface and tangent to the surface.
	The direction of the normal component is reversed, and the components are recomposed into a new velocity heading away from the surface.
	The normal component is multiplied by the resilience parameter and the tangential component, if its magnitude is greater than cutoff, is multiplied by (1 - the friction parameter) when being composed into the new velocity vector.
	The cutoff parameter can allow particles to glide smoothly along a surface without sticking.
	NOTES
		Since particles are tested to see whether they would pass through the domain if Move were called now, it is best to have pBounce be the last action that modifies a particles velocity before Move.
		Also, actions such as pRandomDisplace and pVortex that modify a particles position directly, rather than modifying its velocity vector, may yield unsatisfying results when used with Bounce.
		At present the only domains for which Bounce is implemented are PDSphere, PDRectangle, DTriangle, PDDisc and PDPlane.
		For spheres, the particle is always forced out of the sphere.
		For planes, triangles and discs, the particles bounce off either side of the surface.
		For rectangles, particles bounce off either side of the diamond-shaped patch whose corners are o, o+u, o+u+v, and o+v.
	Bounce doesnt work correctly with small time step sizes for particle sliding along a surface.
	The friction and resilience parameters should not be scaled by dt, since a bounce happens instantaneously.
	On the other hand, they should be scaled by dt because particles sliding along a surface will hit more often if dt is smaller.
	'    !

DomainParticleAction subclass: #DomainRandomAcceleration
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #DomainRandomAcceleration) comment: '
	Accelerate particles in random directions.
	For each particle, chooses an acceleration vector from the specified domain and adds it to the particle velocity.
	Reducing the time step, dt, will make a higher probability of being near the original velocity after unit time.
	Smaller dt approach a normal distribution of velocity vectors instead of a square wave distribution.
	'   !

DomainParticleAction subclass: #DomainRandomDisplace
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #DomainRandomDisplace) comment: '
	Immediately replace position with a position from the domain.
	For each particle, chooses a displacement vector from the specified domain and adds it to the particles position.
	Reducing the time step, dt, will make a higher probability of being near the original position after unit time.
	Smaller dt approach a normal distribution of particle positions instead of a square wave distribution.
	'  !

DomainParticleAction subclass: #DomainRandomVelocity
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #DomainRandomVelocity) comment: '
	Immediately replace velocity with a velocity from the domain.
	For each particle, sets the particles velocity vector to a random vector in the specified domain.
	' !

DomainParticleAction subclass: #DomainSink
  instanceVariableNames: 
    ' killInside '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #DomainSink) comment: '
    Kill particles with positions on wrong side of the domain.
    If killInside is true, deletes all particles inside the given domain.
    If killInside is false, deletes all particles outside the given domain.
    '    !

DomainSink subclass: #DomainSinkInside
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''  !

DomainSink subclass: #DomainSinkOutside
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: '' !

DomainParticleAction subclass: #DomainSource
  instanceVariableNames: 
    ' rate '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #DomainSource) comment: '
	Add particles in the specified domain.
	Adds new particles to the current particle group.
	The particle positions are chosen from the given domain.
	The particle colors, sizes, initial ages, velocities, and secondary positions are chosen according to their current domains.
	Rate is the number of particles to add per unit time.
	If rate / dt is a fraction then DomainSource adjusts the number of particles to add during this time step so that the average number added per unit time is rate.
	'  !

SimpleParticleAction subclass: #PrimitiveParticleAction
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: '' !

PrimitiveParticleAction subclass: #ParticleCoAction
  instanceVariableNames: 
    ' magnitude epsilon maxRadius '
  classVariableNames: ''
  poolDictionaries: ''  !

ParticleCoAction subclass: #ParticleFollow
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleFollow) comment: '
	Accelerate toward the next particle in the group.
	This allows snaky effects where the particles follow each other.
	Each particle is accelerated toward the next particle in the group by an amount proportional to magnitude.
	As with most acceleration actions, the amount of acceleration falls off inversely with r2.
	But when r is small, the acceleration would be infinite, so epsilon is always added to r.
	maxRadius defines the sphere of influence of this action.
	No particle further than maxRadius from its predecessor is affected.
	NOTES
		The Follow action does not affect the last particle in the group.
		This allows controlled effects where the last particle in the group is killed after each time step and replaced by a new particle at a slightly different position.
		See KillOld to learn how to kill the last particle in the group after each step.
	' !

ParticleCoAction subclass: #ParticleGravitate
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleGravitate) comment: '
	Accelerate each particle toward each other particle.
	Each particle is accelerated toward each other particle in the group by an amount proportional to magnitude.
	As with most acceleration actions, the amount of acceleration falls off inversely with r2.
	But when r is small, the acceleration would be infinite, so epsilon is always added to r.
	maxRadius defines the sphere of influence of this action.
	No particle further than maxRadius from another particle is affected.
	NOTES
		This action is more computationally intensive than the others are because each particle is affected by each other particle.
	' !

ParticleCoAction subclass: #ParticleMatchVelocity
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleMatchVelocity) comment: '
	Modify each particle velocity to be similar to that of its neighbors.
	Each particle is accelerated toward the weighted mean of the velocities of other particles in the group by an amount proportional to magnitude.
	As with most acceleration actions, the amount of acceleration falls off inversely with r2 to the other particles.
	But when r is small, the acceleration would be infinite, so epsilon is always added to r.
	Using an epsilon similar in magnitude to magnitude can increase the range of influence of nearby particles on this particle.
	NOTES:
		This action is more computationally intensive than the others are because each particle is affected by each other particle.
	'  !

ParticleCoAction subclass: #PointParticleCoAction
  instanceVariableNames: 
    ' x y z '
  classVariableNames: ''
  poolDictionaries: ''  !

PointParticleCoAction subclass: #ParticleOrbitLine
  instanceVariableNames: 
    ' axisX axisY axisZ '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleOrbitLine) comment: '
	Accelerate toward the closest point on the given line.
	point and axis define an infinite line, where point can be any point on the line and axis is any vector parallel to the line.
	For each particle, this action computes the vector to the closest point on the line, and accelerates the particle in the vector direction.
	The amount of acceleration applied is directly proportional to magnitude.
	As with most acceleration actions, the amount of acceleration falls off inversely with r2.
	But when r is small, the acceleration would be infinite, so epsilon is always added to r.
	maxRadius defines the infinite cylinder of influence of this action.
	No particle further than maxRadius from the line is affected.
	'   !

PointParticleCoAction subclass: #ParticleOrbitPoint
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleOrbitPoint) comment: '
	Accelerate toward the given center point.
	For each particle, this action computes the vector to the center point, and accelerates the particle in the vector direction.
	The amount of acceleration applied is directly proportional to magnitude. 
	As with most acceleration actions, the amount of acceleration falls off inversely with r2.
	But when r is small, the acceleration would be infinite, so epsilon is always added to r.
	maxRadius defines the infinite cylinder of influence of this action.
	No particle further than maxRadius from the line is affected.
	'   !

PrimitiveParticleAction subclass: #ParticleCopyVertexB
  instanceVariableNames: 
    ' copyVelocity copyPosition '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleCopyVertexB) comment: '
	Set the secondary position from current position.
	If copyPosition is true, sets the positionB of each particle in the current particle group to be equal to the current position of that particle.
	This makes each particle "remember" this position so it can later return to it using Restore.
	If copyVelocity is true, sets the velocityB of each particle in the current particle group to be equal to the current velocity of that particle.
	This is useful for computing the orientation of the particle when rendering it using DrawGroup.
	'   !

PrimitiveParticleAction subclass: #ParticleKillOld
  instanceVariableNames: 
    ' ageLimit killLessThan '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleKillOld) comment: '
	Remove old particles.
	Removes all particles older than ageLimit.
	But if killLessThan is true, it instead removes all particles newer than ageLimit.
	ageLimit is not clamped, so negative values are ok.
	This can be used in conjunction with StartingAge(-n) to create and then kill a particular set of particles.
	' !

ParticleKillOld subclass: #ParticleKillYoung
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''    !

PrimitiveParticleAction subclass: #ParticleMove
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleMove) comment: '
	Move particle positions based on velocities.
	This action actually updates the particle positions by adding the current velocity to the current position.
	This is typically the last particle action performed in an iteration of a particle simulation, and typically only occurs once per action list.
	The velocity is multiplied by the time step length, before being added to the position.
	This implements Eulers method of numerical integration with a constant, but specifiable step size.
	' !

PrimitiveParticleAction subclass: #ParticleRestore
  instanceVariableNames: 
    ' timeLeft '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleRestore) comment: '
	Over time, restore particles to their secondary positions.
	Computes a new velocity for each particle that will make the particle arrive at its positionB at the specified amount of time in the future.
	The curved path that the particles take is a parametric quadratic.
	Once the specified amount of time has passed, the particles are clamped to their positionB and their velocities are set to 0 to freeze them in place.
	If ParticleRestore is used in immediate mode, it is the applications responsibility to decrease timeLeft by dt on each use.
	When in an action list, time_left gets decremented automatically.
	The positionB attribute of each particle is typically the particles position when it was created, or it can be specified within a domain.
	This is controlled by ParticleVertexBTracks, PaerticleVertexB, and ParticleVertexBD.
	The positionB can be set at any time to the particles current position using the ParticleCopyVertexB action.
	'   !

PrimitiveParticleAction subclass: #ParticleSpeedLimit
  instanceVariableNames: 
    ' minSpeed maxSpeed '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleSpeedLimit) comment: '
	Clamp each particles speed to the given min and max.
	Computes each particles speed (the magnitude of its velocity vector) and if it is less than minSpeed or greater than maxSpeed it is clamped to those bounds, while preserving the velocity vectors direction.
	'  !

PrimitiveParticleAction subclass: #ParticleTargetColor
  instanceVariableNames: 
    ' red green blue alpha scale '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleTargetColor) comment: '
	Change color of all particles toward the specified color.
	Modifies the color and alpha of each particle to be scale percent of the way closer to the specified color and alpha.
	scale is multiplied by dt before scaling the sizes.
	Thus, using smaller dt causes a slightly faster approach to the target color.
	'    !

PrimitiveParticleAction subclass: #PointParticleAction
  instanceVariableNames: 
    ' x y z '
  classVariableNames: ''
  poolDictionaries: '' !

PointParticleAction subclass: #ParticleDamping
  instanceVariableNames: 
    ' low high '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleDamping) comment: '
    Simulate air by slowing down particle velocities.
    If a particles velocity magnitude is within low and high, then multiply each component of the velocity by the respective damping constant.
    Typically, the three components of damping will have the same value.
    NOTES
        There are no bounds on the damping constants.
        Thus, by giving values greater than 1.0 they may be used to speed up particles in stead of slow them down.
    '  !

PointParticleAction subclass: #ParticleExplosion
  instanceVariableNames: 
    ' velocity magnitude stdev epsilon age '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleExplosion) comment: '
	An Explosion.
	Causes an explosion by accelerating all particles away from the center.
	Particles are accelerated away from the center by an amount proportional to magnitude.
	As with most acceleration actions, the amount of acceleration falls off inversely with r2.
	But when r is small, the acceleration would be infinite, so epsilon is always added to r.
	The shock wave of the explosion has a gaussian magnitude.
	The center of the gaussian travels outward from the center at the specified velocity.
	So at a given time step, particles at a distance (velocity * age) from center will receive the most acceleration, and particles not at the peak of the shock wave will receive a lesser acceleration.
	The shock wave has a standard deviation of stdev, which is the sharpness or broadness of the strength of the wave.
	age is used to calculate the radius of the shock wave.
	For Explosion in action lists, age is the initial age of the explosion.
	It is incremented by dt after each evaluation.
	For immediate mode, age is the current age of the explosion, and it is up to the application to increment the age parameter for each Explosion.
	' !

PointParticleAction subclass: #ParticleGravity
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleGravity) comment: '
	Accelerate particles in the given direction.
	The gravity acceleration vector is simply added to the velocity vector of each particle at each time step.
	The magnitude of the gravity vector is the acceleration due to gravity.
	For example, pGravity(0, 0, -9.8) specifies gravity in the negative Z direction.
	'   !

PointParticleAction subclass: #ParticleJet
  instanceVariableNames: 
    ' magnitude epsilon maxRadius '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleJet) comment: '
    Accelerate particles that are near the center of the jet.
	For each particle, chooses an acceleration vector from the domain and applies it to the particles velocity.
	The amount of acceleration applied is directly proportional to magnitude.
	As with most acceleration actions, the amount of acceleration falls off inversely with r2.
	But when r is small, the acceleration would be infinite, so epsilon is always added to r.
	The domain from which acceleration vectors are chosen is the current velocity domain.
	maxRadius defines the sphere of influence of this action.
	No particle further than maxRadius from the center is affected.
    '  !

ParticleJet subclass: #ParticleVortex
  instanceVariableNames: 
    ' axisX axisY axisZ '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleVortex) comment: '
    Swirl particles around a vortex.
	center and axis define an infinite line, where center represents the tip of the vortex and axis is a vector along the line, the length of which is irrelevant.
	As with most acceleration actions, the amount of acceleration falls off inversely with r2 to the center.
	But when r is small, the acceleration would be infinite, so epsilon is always added to r.
	Using an epsilon similar in magnitude to magnitude can increase the range of influence of the vortex.
	maxRadius defines the sphere of influence of this action.
	No particle further than maxRadius from the vortex center is affected.
    '   !

PointParticleAction subclass: #ParticleTargetSize
  instanceVariableNames: 
    ' scaleX scaleY scaleZ '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleTargetSize) comment: '
	Change sizes of all particles toward the specified size.
	Modifies the size of each particle to be scale percent of the way closer to the specified size triple.
	This makes sizes grow asymptotically closer to the given size.
	scale is multiplied by dt before scaling the sizes.
	Thus, using smaller dt causes a slightly faster approach to the target size.
	The separate scales for each component allow only selected components to be scaled.
	'  !

PointParticleAction subclass: #ParticleTargetVelocity
  instanceVariableNames: 
    ' scale '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleTargetVelocity) comment: '
	Change velocity of all particles toward the specified velocity.
	Modifies the velocity of each particle to be scale percent of the way closer to the specified velocity triple.
	This makes velocities grow asymptotically closer to the given velocity.
	scale is multiplied by dt before scaling the velocities.
	Thus, using smaller dt causes a slightly faster approach to the target velocity.
	'  !

PointParticleAction subclass: #ParticleVertex
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleVertex) comment: '
	Add a single particle at the specified location.
	'   !

ParticleAction subclass: #SimpleStateSetting
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #SimpleStateSetting) comment: '
	With the exception of ParticleTimeStep these state settings dictate the properties of particles to be created by Source or Vertex actions.
	'  !

SimpleStateSetting subclass: #DomainStateSetting
  instanceVariableNames: 
    ' domain '
  classVariableNames: ''
  poolDictionaries: ''  !

DomainStateSetting subclass: #DomainColor
  instanceVariableNames: 
    ' alpha '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #DomainColor) comment: '
	Specify the color domain of new particles.
	All new particles will have a color chosen randomly from within the specified domain
	 and will have an alpha value (usually used for transparency) of alpha.
	'  !

DomainStateSetting subclass: #DomainSize
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #DomainSize) comment: '
	Specify the size domain of new particles.
	All new particles will have a size chosen randomly from within the specified domain.
	The size values may be negative.
	The size does not affect any particle dynamics, including acceleration and bouncing.
	It is merely a triple of rendering attributes, like color, and can be interpreted.
	In particular, the three components do not need to be used together as three dimensions of the particle size.
	For example, one could be interpreted as radius, another as length, and another as density.
	'   !

DomainStateSetting subclass: #DomainVelocity
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #DomainVelocity) comment: '
	Specify the initial velocity domain of new particles.
	'   !

DomainStateSetting subclass: #DomainVertexB
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #DomainVertexB) comment: '
	Specify the secondary position domain of new particles.
	Set the domain from which the positionB of each particle will be randomly chosen when it is created.
	But if VertexBTracks is set to true (the default), then the positionB will instead be the same as the initial particle position.
	' !

SimpleStateSetting subclass: #PrimitiveStateSetting
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: '' !

PrimitiveStateSetting subclass: #ParticleColor
  instanceVariableNames: 
    ' red green blue alpha '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleColor) comment: '
	Specify the color of new particles.
	All new particles will have the color red, green, blue, alpha.
	'   !

PrimitiveStateSetting subclass: #ParticleCurrentGroup
  instanceVariableNames: 
    ' group '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleCurrentGroup) comment: '
	Specify the current group.
	'  !

PrimitiveStateSetting subclass: #ParticleStartingAge
  instanceVariableNames: 
    ' age stdev '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleStartingAge) comment: '
	Specify the initial age of new particles.
	The value age can be positive, zero, or negative.
	Giving particles different starting ages allows Kill mechanism to distinguish between which to kill in interesting ways.
	Setting stdev to a non-zero value will give the particles an initial age with a normal distribution with mean age and standard deviation stdev.
	When many particles are created at once this allows a few particles to die at each time step, yielding a more natural effect.
	'    !

PrimitiveStateSetting subclass: #ParticleTimeStep
  instanceVariableNames: 
    ' dt '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleTimeStep) comment: '
	Specify the time step length.
	The Particle System uses a discrete time approximation to all actions.
	This means that actions are applied to the particles at a particular instant in time as if the actions effect accumulated over a small time interval, dt, with the world being constant over the interval.
	The clock is then "ticked" by the length of the interval and the actions can then be reapplied with the particles having their updated values.
	This is the standard method of doing almost all time-varying simulations in computer science.
	How does the time step, dt, relate to the applications frame rate?
	The easiest method is to apply the actions once per frame.
	If the application prefers to keep time in terms of seconds, dt can be set to (1 / frames_per_second).
	But more often, it is easier for a time unit to be one frame instead of one second.
	In this case, dt should be 1.0, which is the default.
	For higher quality, the application can apply particle actions more than once per frame.
	This provides smoother, more realistic results in many subtle ways.
	Suppose the application wanted to compute three animation steps for each rendered frame.
	Set dt to 1/3 its previous value using ParticleTimeStep, then loop three times over all the action code that gets executed per frame, including ParticleMove.
	If using action lists, this can be simply a loop over the CallActionList.
	The run-time results should be about the same, but with fewer discrete approximation artifacts.
	Surprisingly enough, the actions themselves usually take a small percentage of the CPU time, so using a finer step size usually does not impact frame rate very much.
	In terms of numerical integration, particle actions can be thought of as the first derivative of unknown functions dictating the particle attributes (such as position) over time.
	In order to compute the particle attributes these derivative functions must be integrated.
	Since closed form integration doesnt make sense for most actions, Eulers method is used instead.
	Eulers method is simply the method just described the evaluation of the derivative functions at a particular time and then incrementing the current particle values by these derivative results times dt.
	In Eulers method, the smaller the dt, the more accurate the results.
	NOTES
		Unlike with other state setting, action lists execute using the current dt value set by ParticleTimeStep,
			rather than the time step value that was current when the action list was created.
		Making action lists independent of time step size allows the time step to be changed without recompiling the action list.
	'   !

PrimitiveStateSetting subclass: #ParticleVertexBTracks
  instanceVariableNames: 
    ' doCopy '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleVertexBTracks) comment: '
	Specify how the secondary position of each new particle is chosen.
	Specify how the positionB of each new particle emitted will be chosen.
	The positionB is the destination position or initial position of particles.
	If doCopy is true then when a particle is created its positionB will be the same as its initial particle position (the default).
	If doCopy is false then when a particle is created its positionB will be chosen from the VertexB domain.
	'  !

PrimitiveStateSetting subclass: #PointStateSetting
  instanceVariableNames: 
    ' x y z '
  classVariableNames: ''
  poolDictionaries: '' !

PointStateSetting subclass: #ParticleSize
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleSize) comment: '
	Specify the size of new particles.
	All new particles will have a size of size. The size values may be negative.

	The size does not affect any particle dynamics, including acceleration and bouncing.
	It is merely a triple of rendering attributes, like color, and can be interpreted.
	In particular, the three components do not need to be used together as three dimensions of the particle size.
	For example, one could be interpreted as radius, another as length, and another as density.
	'    !

PointStateSetting subclass: #ParticleVelocity
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleVelocity) comment: '
	Specify the initial velocity of new particles.
	'   !

PointStateSetting subclass: #ParticleVertexB
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleVertexB) comment: '
	Specify the secondary position of new particles.
	'   !

ParticleObject subclass: #ParticleDomain
  instanceVariableNames: 
    ' a0 a1 a2 a3 a4 a5 a6 a7 a8 '
  classVariableNames: ''
  poolDictionaries: 
    ' ParticleConstants '.

  (Smalltalk at: #ParticleDomain) comment: '
	A Domain is a representation of a region of space.
	For example, the ParticleSource action uses a domain to describe the volume in which a particle will be created.
	A random point within the domain is chosen as the initial position of the particle.
	The ParticleAvoid, ParticleSink and ParticleBounce actions use domains to describe a volume in space for particles to steer around, die when they enter, or bounce off, respectively.
	Domains are also used to describe velocities.
	Picture the velocity vector as having its tail at the origin and its tip being in the domain.
	Finally, domains can be used to describe colors. 

	'    !

ParticleDomain subclass: #CenteredDomain
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #CenteredDomain) comment: '
	Domain with an origin.
	'  !

CenteredDomain subclass: #BoxDomain
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #BoxDomain) comment: '
	These are the minima and maxima of an axis-aligned box.
	It doesnt matter which of each coordinate is min and which is max.
	'  !

CenteredDomain subclass: #PlaneDomain
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #PlaneDomain) comment: '
	Domain for points in a plane.
	The point o is a point on the plane.	n is the normal vector of the plane.
	It need not be unit length.
	If you have a plane in a,b,c,d form remember that n = [a,b,c]
		and you can compute a suitable point o as o = -n*d.
	The normal will get normalized, so it need not already be normalized.
	'    !

PlaneDomain subclass: #DiscDomain
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #DiscDomain) comment: '
	The point x, y, z is the center of a disc in the plane with normal nx, ny, nz.
	The disc has outer radius radius1 and inner radius radius2.
	The normal will get normalized, so it need not already be normalized.
	'   !

CenteredDomain subclass: #RectangleDomain
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #RectangleDomain) comment: '
	The point o is a point on the plane.
	u and v are (non-parallel) basis vectors in the plane.
	They dont need to be normal or orthogonal.
	'    !

CenteredDomain subclass: #SphereDomain
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #SphereDomain) comment: '
	Points inside a sphere (and elipse).
	'    !

ParticleDomain subclass: #ConeDomain
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ConeDomain) comment: '
	Points in a Conical shell.
	The first point is the apex of the cone and the second is the other endpoint of the axis of the cone.
	radius1 is the radius of the base of the cone.
	radius2 is the radius of the base of a cone to subtract from the first cone to create a conical shell.
	This is similar to the cylindrical shell, which can be thought of as a large cylinder with a smaller cylinder subtracted from the middle. Both cones share the same apex and axis, which implies that the thickness of the conical shell tapers to 0 at the apex. radius2 = 0 for a solid cone with no empty space in the middle.
	'    !

ParticleDomain subclass: #LineDomain
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #LineDomain) comment: '
	These are the endpoints of a line segment.
	'  !

LineDomain subclass: #CylinderDomain
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #CylinderDomain) comment: '
	Points in a cylindrical shell.
	The two points are the endpoints of the axis of the cylinder.
	radius1 is the outer radius, and radius2 is the inner radius for a cylindrical shell.
	radius2 = 0 for a solid cylinder with no empty space in the middle.
	'    !

ParticleDomain subclass: #PointDomain
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #PointDomain) comment: '
	This domain is a single point.
	'    !

PointDomain subclass: #BlobDomain
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #BlobDomain) comment: '
	The point x, y, z is the center of a normal probability density of standard deviation stdev.
	The density is radially symmetrical.
	The blob domain allows for some very natural-looking effects because there is no sharp, artificial-looking boundary at the edge of the domain.
	'   !

ParticleDomain subclass: #TriangleDomain
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #TriangleDomain) comment: '
	These are the vertices of a triangle.
	The triangle can be used to define an arbitrary geometrical model for particles to bounce off, or generate particles on its surface (and explode them), etc.
	'    !

ParticleObject subclass: #ParticleExample
  instanceVariableNames: 
    ' group drawCallBack preRenderAction defaultBitmap '
  classVariableNames: ''
  poolDictionaries: ''   !

ParticleExample subclass: #ParticleNormalizedExample
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''    !

ParticleNormalizedExample subclass: #Balloons
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''   !

ParticleNormalizedExample subclass: #Explosion
  instanceVariableNames: 
    ' age numSteps '
  classVariableNames: ''
  poolDictionaries: ''  !

ParticleNormalizedExample subclass: #FireFlies
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''  !

ParticleNormalizedExample subclass: #Fountain
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #Fountain) comment: '
	A fountain spraying up in the middle of the screen.
	'  !

ParticleNormalizedExample subclass: #JetExample
  instanceVariableNames: 
    ' jetX jetY jetZ djx djy djz randomizer '
  classVariableNames: ''
  poolDictionaries: ''    !

JetExample subclass: #JetSpray
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''  !

JetExample subclass: #Snake
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: '' !

JetExample subclass: #Swirl
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: '' !

Swirl subclass: #Chaos
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''  !

Swirl subclass: #Disperse
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''   !

Swirl subclass: #Vortex
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: '' !

ParticleNormalizedExample subclass: #ParticleAtom
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleAtom) comment: '
	Particles orbiting an atomic center.
	' !

ParticleNormalizedExample subclass: #Rain
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''   !

ParticleNormalizedExample subclass: #Shape
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''  !

ParticleNormalizedExample subclass: #Waterfall
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #Waterfall) comment: '
	A waterfall pouring down from above.
	'   !

ParticleObject subclass: #ParticleExampleEditor
  instanceVariableNames: 
    ' demo example polygon '
  classVariableNames: ''
  poolDictionaries: '' !

ParticleObject subclass: #ParticleHandle
  instanceVariableNames: 
    ' handle '
  classVariableNames: ''
  poolDictionaries: ''  !

ParticleHandle subclass: #ParticleGroup
  instanceVariableNames: 
    ' drawCallBack preDrawActions '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleGroup) comment: '
	A particle group is a system of particles that are all acted on together. 
	It can be considered a dynamic geometric object. 
	' !

ParticleObject subclass: #ParticleStructure
  instanceVariableNames: 
    ' structure '
  classVariableNames: ''
  poolDictionaries: ''    !

ParticleStructure subclass: #Particle
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #Particle) comment: '
	For the purposes of the Particle System, a particle is not just a very tiny speck.
	Particles can be drawn as anything - water droplets, birds, tumbleweeds, even people.
	The Particle System is generally useful for operating on many similar objects that all move and change according to the same basic rules, no matter what the objects and rules are.
	A particle in the abstract sense is merely an entity with a small set of attributes such as position and color that ultimately dictate the particle behavior and appearance.
	Attributes:
		The position (aVector), tells where the particle is.
		The color is represented as aVector.
		The velocity consists of aVector telling the direction and speed (velocity vector) of the particle movement.
		The size is aVector representing how large the particle is for purposes of drawing.
			At present, the size attribute does not affect the particle behavior (under gravity, for example), and should not be interpreted as mass in particle physics equations.
			The size also does not indicate how close the particle is to other particles or surfaces.
			It is only a drawing attribute.
		The age is the amount of time since the particle creation.
		The secondary position, or positionB, specifies another position in space for this particle.
			It usually represents the particle initial position, or the particle destination. 
	'  !

ParticleStructure subclass: #ParticlesArray
  instanceVariableNames: 
    ' count '
  classVariableNames: ''
  poolDictionaries: ''    !

ParticleStructure subclass: #ParticleVector
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: '' !

DynalinkUnderTest variableByteSubclass: #ParticleDLL
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParticleDLL) comment: '
	@2000 Particle Interface
	Smalltalk interface to the Particles API of David K. McAllister
	http://www.cs.unc.edu/~davemc/Particle
	Implemented by Alejandro Reimondo.  [aleReimondo@smalltalking.net]
	'    !


!ParticleObject class methods !
  
dll
	" Private - Returns the current library in use.
	We can't use #library as the name of this message because #library express SLL support.
	"

	^ParticleDLL library! !



!ParticleObject methods !

cppFloat: aNumber
	" Private - Transform aNumber into a cpp float parameter. "

	aNumber isNumber ifFalse: [ "aNumber can be an evaluable object."
		^self dll cppFloat: aNumber evaluate
	].
	^self dll cppFloat: aNumber!
   
dll
	" Private - Returns the current library in use. "

	^self class dll!

epsilon
	" Private - Returns the epsilon value. "

	^self dll epsilon!
   
maxFloat
	" Private - Returns the maxFloat value. "

	^self dll maxFloat!

maxInt
	" Private - Returns the maxInt value. "

	^self dll maxInt!
  
stFloat: aByteArray
	" Private - Transform aByteArray to an Smalltalk Float. "

	| result |
	result := OLEStructure internalFloatFromExternalFloat: aByteArray.
	Float clearExceptionFlags. "clear exception flags because sometimes are signalled incorrectly."
	^result!

toDo
	" Debug - Only used for code to be revised.

	Smalltalk sendersOf: #toDo

	"! !



!ParticleAction class methods !
   
blobAt: origin stdev: stdev
	"Returns an instance of the receiver on aPlaneDomain."

	^self forDomain: (BlobDomain origin: origin stdev: stdev)!
 
boxOrigin: origin corner: corner
	"Returns an instance of the receiver on aBoxDomain."

	^self forDomain: (BoxDomain origin: origin corner: corner)!
 
compositeClass
	" Private - Returns the class to use to build composite instances."

	^CompositeParticleAction!
  
cylinderFrom: origin to: destination radius: radius
	"Returns an instance of the receiver on aCylinderDomain."

	^self forDomain: (CylinderDomain new
		origin: origin;
		destination: destination;
		radius: radius;
		yourself)!
   
discAt: origin normal: normal radius: radius
	"Returns an instance of the receiver on aDiscDomain."

	^self forDomain: (DiscDomain new
		origin: origin;
		normal: normal;
		radius: radius;
		yourself)!

empty
	" Returns an empty composite action."

	^self compositeClass empty!
   
forDomain: aDomain
	"Returns an instance of the receiver on aDomain."

	^self invalidMessage!

from: source to: destination
	"Returns an instance of the receiver on aLineDomain from source to destination."

	^self lineFrom: source to: destination!
 
lineFrom: source to: destination
	"Returns an instance of the receiver on aLineDomain from source to destination."

	^self forDomain: (LineDomain from: source to: destination)!
 
origin: origin corner: corner
	"Returns an instance of the receiver on aBoxDomain."

	^self forDomain: (BoxDomain origin: origin corner: corner)!

planeAt: origin normal: normal
	"Returns an instance of the receiver on aPlaneDomain."

	^self forDomain: (PlaneDomain origin: origin normal: normal)!
   
pointAt: origin
	"Returns an instance of the receiver on aPointDomain."

	^self forDomain: (PointDomain new
		origin: origin;
		yourself)!
 
rectangleAt: anArray u: uArray v: vArray
	"Returns an instance of the receiver on aRectangleDomain."

	^self forDomain: (RectangleDomain origin: anArray u: uArray v: vArray)!
   
sphereAt: origin radius: radius
	"Returns an instance of the receiver on aSphereDomain."

	^self forDomain: (SphereDomain new
		origin: origin;
		radius: radius;
		yourself)! !



!ParticleAction methods !
 
, anAction
	"Returns a composite action with the receiver an anAction."

	^self compositeClass with: self with: anAction!

compositeClass
	" Private - Returns the class to use to build composite instances."

	^self class compositeClass!

isComposite
	"Returns true if the receiver is Composite."

	^false!
  
isSimpleAction
	"Returns true if the receiver is a Simple Action."

	^false!
 
value
	" The receiver must perform it's action. "

	^self implementedBySubclass! !



!CompositeParticleAction class methods !
   
empty
	" Returns an empty composite action."

	^self new!

with: anAction
	" Returns an instance of the receiver. "

	^self new
		add: anAction;
		yourself!
  
with: aAction with: bAction
	" Returns an instance of the receiver. "

	^self new
		add: aAction;
		add: bAction;
		yourself!
 
with: aAction
	with: bAction
	with: cAction
	" Returns an instance of the receiver. "

	^self new
		add: aAction;
		add: bAction;
		add: cAction;
		yourself!
  
with: aAction
	with: bAction
	with: cAction
	with: dAction
	" Returns an instance of the receiver. "

	^self new
		add: aAction;
		add: bAction;
		add: cAction;
		add: dAction;
		yourself!
 
withAll: actions
	" Returns an instance of the receiver. "

	^self new
		addAll: actions;
		yourself! !



!CompositeParticleAction methods !
  
, anAction
	" Append anAction to the receiver.
	Returns the receiver. "

	self add: anAction.!
  
add: anAction
	" Private - Adds anAction to the receiver. "

	^self elements add: anAction!
  
addAll: actions
	" Private - Adds actions to the receiver. "

	actions do: [:each| self add: each ].
	^actions!
 
elements
    "Returns the instance var of the receiver
     for external manipulation"

	elements isNil ifTrue: [ self initializeElements ].
    ^elements!

initializeElements
	" Private - Initializes the receiver's elements"

	elements := OrderedCollection new.!
   
isComposite
	"Returns true if the receiver is Composite."

	^true!
   
value
	" The receiver must perform it's action. "

	self elements do: [:each| each value ].! !



!ParticleActionList methods !
  
add: anAction
	" Private - Adds anAction to the receiver. "

	(anAction isKindOf: self species)
	ifTrue: [
		"Action list can't be recursive, so we will internalize its elements."
		self addAll: anAction elements.
		^anAction
	].
	anAction isSimpleAction ifFalse: [
		^self error: 'Must be a simple Action.'
	].

	^super add: anAction! !



!SimpleParticleAction methods !
   
isSimpleAction
	"Returns true if the receiver is a Simple Action."

	^true! !



!DomainParticleAction class methods !
   
forDomain: aDomain
	"Returns an instance of the receiver on aDomain."

	^self new
		domain: aDomain;
		yourself!
   
on: aDomain
	"Returns an instance of the receiver actin on aDomain."

	^self forDomain: aDomain! !



!DomainParticleAction methods !

domain
    "Returns the instance var of the receiver
     for external manipulation"

	domain isNil ifTrue: [ self initializeDomain ].
    ^domain!

domain: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    domain := anObject!
   
domainClass
	" Private - Returns the class used to create the receiver's domain.
	This message must be reimplemented by classes that support default initialization of domains.
	If this method has not been reimplemented, you must set the domain to use this action."

	^self invalidMessage!
   
initializeDomain
	" Private - Initializes the receiver's domain"

	domain := self domainClass new.! !



!DomainAvoid methods !
  
epsilon
    "Returns the instance var of the receiver
     for external manipulation"
    ^epsilon!
  
epsilon: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    epsilon := anObject!
 
lookAhead
    "Returns the instance var of the receiver
     for external manipulation"
    ^lookAhead!
  
lookAhead: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    lookAhead := anObject!
 
magnitude
    "Returns the instance var of the receiver
     for external manipulation"
    ^magnitude!
  
magnitude: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    magnitude := anObject!
 
value
	" The receiver must perform it's action. "

	self dll
		pAvoid: (self cppFloat: self magnitude) asParameter
		epsilon: (self cppFloat: self epsilon) asParameter
		lookAhead: (self cppFloat: self lookAhead) asParameter
		dtype: self domain type asParameter
		a0: (self cppFloat: self domain a0) asParameter
		a1: (self cppFloat: self domain a1) asParameter
		a2: (self cppFloat: self domain a2) asParameter
		a3: (self cppFloat: self domain a3) asParameter
		a4: (self cppFloat: self domain a4) asParameter
		a5: (self cppFloat: self domain a5) asParameter
		a6: (self cppFloat: self domain a6) asParameter
		a7: (self cppFloat: self domain a7) asParameter
		a8: (self cppFloat: self domain a8) asParameter! !



!DomainBounce methods !
  
cutoff
    "Returns the instance var of the receiver
     for external manipulation"

	cutoff isNil ifTrue: [ self initializeCutoff ].
    ^cutoff!

cutoff: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    cutoff := anObject!
   
friction
    "Returns the instance var of the receiver
     for external manipulation"

	friction isNil ifTrue: [ self initializeFriction ].
    ^friction!

friction: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    friction := anObject!
   
initializeCutoff
	" Private - Initializes the receiver's cutoff"

	cutoff := 0.!
 
initializeFriction
	" Private - Initializes the receiver's friction"

	friction := 0.!
   
resilience
    "Returns the instance var of the receiver
     for external manipulation"
    ^resilience!

resilience: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    resilience := anObject!
   
value
	" The receiver must perform it's action. "

	self dll
		pBounce: (self cppFloat: self friction) asParameter
		resilience: (self cppFloat: self resilience) asParameter
		cutoff: (self cppFloat: self cutoff) asParameter
		dtype: self domain type asParameter
		a0: (self cppFloat: self domain a0) asParameter
		a1: (self cppFloat: self domain a1) asParameter
		a2: (self cppFloat: self domain a2) asParameter
		a3: (self cppFloat: self domain a3) asParameter
		a4: (self cppFloat: self domain a4) asParameter
		a5: (self cppFloat: self domain a5) asParameter
		a6: (self cppFloat: self domain a6) asParameter
		a7: (self cppFloat: self domain a7) asParameter
		a8: (self cppFloat: self domain a8) asParameter! !



!DomainRandomAcceleration methods !
  
value
	" The receiver must perform it's action. "

	self dll
		pRandomAccel: self domain type asParameter
		a0: (self cppFloat: self domain a0) asParameter
		a1: (self cppFloat: self domain a1) asParameter
		a2: (self cppFloat: self domain a2) asParameter
		a3: (self cppFloat: self domain a3) asParameter
		a4: (self cppFloat: self domain a4) asParameter
		a5: (self cppFloat: self domain a5) asParameter
		a6: (self cppFloat: self domain a6) asParameter
		a7: (self cppFloat: self domain a7) asParameter
		a8: (self cppFloat: self domain a8) asParameter! !



!DomainRandomDisplace methods !
  
value
	" The receiver must perform it's action. "

	self dll
		pRandomDisplace: self domain type asParameter
		a0: (self cppFloat: self domain a0) asParameter
		a1: (self cppFloat: self domain a1) asParameter
		a2: (self cppFloat: self domain a2) asParameter
		a3: (self cppFloat: self domain a3) asParameter
		a4: (self cppFloat: self domain a4) asParameter
		a5: (self cppFloat: self domain a5) asParameter
		a6: (self cppFloat: self domain a6) asParameter
		a7: (self cppFloat: self domain a7) asParameter
		a8: (self cppFloat: self domain a8) asParameter! !



!DomainRandomVelocity methods !
   
value
	" The receiver must perform it's action. "

	self dll
		pRandomVelocity: self domain type asParameter
		a0: (self cppFloat: self domain a0) asParameter
		a1: (self cppFloat: self domain a1) asParameter
		a2: (self cppFloat: self domain a2) asParameter
		a3: (self cppFloat: self domain a3) asParameter
		a4: (self cppFloat: self domain a4) asParameter
		a5: (self cppFloat: self domain a5) asParameter
		a6: (self cppFloat: self domain a6) asParameter
		a7: (self cppFloat: self domain a7) asParameter
		a8: (self cppFloat: self domain a8) asParameter! !



!DomainSink methods !
 
initializeKillInside
	" Private - Initializes the receiver's killInside"

	killInside := true.!
  
killInside
    "Returns the instance var of the receiver
     for external manipulation"

	killInside isNil ifTrue: [ self initializeKillInside ].
    ^killInside!

killInside: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    killInside := anObject!
   
value
	" The receiver must perform it's action. "

	self dll
		pSink: self killInside asParameter
		type: self domain type asParameter
		a0: (self cppFloat: self domain a0) asParameter
		a1: (self cppFloat: self domain a1) asParameter
		a2: (self cppFloat: self domain a2) asParameter
		a3: (self cppFloat: self domain a3) asParameter
		a4: (self cppFloat: self domain a4) asParameter
		a5: (self cppFloat: self domain a5) asParameter
		a6: (self cppFloat: self domain a6) asParameter
		a7: (self cppFloat: self domain a7) asParameter
		a8: (self cppFloat: self domain a8) asParameter! !



!DomainSinkInside methods !

initializeKillInside
	" Private - Initializes the receiver's killInside"

	killInside := true.! !



!DomainSinkOutside methods !

initializeKillInside
	" Private - Initializes the receiver's killInside"

	killInside := false.! !



!DomainSource methods !

rate
    "Returns the instance var of the receiver
     for external manipulation"
    ^rate!

rate: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    rate := anObject!
   
value
	" The receiver must perform it's action. "

	self dll
		pSource: (self cppFloat: self rate) asParameter
		type: self domain type asParameter
		a0: (self cppFloat: self domain a0) asParameter
		a1: (self cppFloat: self domain a1) asParameter
		a2: (self cppFloat: self domain a2) asParameter
		a3: (self cppFloat: self domain a3) asParameter
		a4: (self cppFloat: self domain a4) asParameter
		a5: (self cppFloat: self domain a5) asParameter
		a6: (self cppFloat: self domain a6) asParameter
		a7: (self cppFloat: self domain a7) asParameter
		a8: (self cppFloat: self domain a8) asParameter! !



!ParticleCoAction methods !
   
epsilon
    "Returns the instance var of the receiver
     for external manipulation"

	epsilon isNil ifTrue: [ self initializeEpsilon ].
    ^epsilon!

epsilon: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    epsilon := anObject!
 
initializeEpsilon
	" Private - Initializes the receiver's epsilon"

	epsilon := super epsilon.!
  
initializeMagnitude
	" Private - Initializes the receiver's magnitude"

	magnitude := 1.!

initializeMaxRadius
	" Private - Initializes the receiver's maxRadius"

	maxRadius := self maxFloat.!

magnitude
    "Returns the instance var of the receiver
     for external manipulation"

	magnitude isNil ifTrue: [ self initializeMagnitude ].
    ^magnitude!

magnitude: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    magnitude := anObject!
 
maxRadius
    "Returns the instance var of the receiver
     for external manipulation"

	maxRadius isNil ifTrue: [ self initializeMaxRadius ].
    ^maxRadius!

maxRadius: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    maxRadius := anObject! !



!ParticleFollow methods !
  
value
	" The receiver must perform it's action. "

	self dll
		pFollow: (self cppFloat: self magnitude) asParameter
		epsilon: (self cppFloat: self epsilon) asParameter
		maxRadius: (self cppFloat: self maxRadius) asParameter! !



!ParticleGravitate methods !
  
value
	" The receiver must perform it's action. "

	self dll
		pGravitate: (self cppFloat: self magnitude) asParameter
		epsilon: (self cppFloat: self epsilon) asParameter
		maxRadius: (self cppFloat: self maxRadius) asParameter! !



!ParticleMatchVelocity methods !
   
value
	" The receiver must perform it's action. "

	self dll
		pMatchVelocity: (self cppFloat: self magnitude) asParameter
		epsilon: (self cppFloat: self epsilon) asParameter
		maxRadius: (self cppFloat: self maxRadius) asParameter! !



!PointParticleCoAction class methods !
 
x: aX y: aY z: aZ
	"Returns an instance of the receiver."

	^self new x: aX; y: aY; z: aZ;yourself! !



!PointParticleCoAction methods !

x
    "Returns the instance var of the receiver
     for external manipulation"
    ^x!
  
x: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    x := anObject!
 
x: aX y: aY z: aZ
    "Sets the receiver's coordinates."

    self x: aX; y: aY; z: aZ!
  
y
    "Returns the instance var of the receiver
     for external manipulation"
    ^y!
  
y: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    y := anObject!
 
z
    "Returns the instance var of the receiver
     for external manipulation"
    ^z!
  
z: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    z := anObject! !



!ParticleOrbitLine methods !
   
axis: aX y: aY z: aZ
    "Sets the receiver's axis."

	self
		axisX: aX;
		axisY: aY;
		axisZ: aZ!

axisX
    "Returns the instance var of the receiver
     for external manipulation"
    ^axisX!
  
axisX: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    axisX := anObject!
 
axisY
    "Returns the instance var of the receiver
     for external manipulation"
    ^axisY!
  
axisY: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    axisY := anObject!
 
axisZ
    "Returns the instance var of the receiver
     for external manipulation"
    ^axisZ!
  
axisZ: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    axisZ := anObject!
 
value
	" The receiver must perform it's action. "

	self dll
		pOrbitLine: (self cppFloat: self x) asParameter
		y: (self cppFloat: self y) asParameter
		z: (self cppFloat: self z) asParameter
		axisX: (self cppFloat: self axisX) asParameter
		axisY: (self cppFloat: self axisY) asParameter
		axisZ: (self cppFloat: self axisZ) asParameter
		magnitude: (self cppFloat: self magnitude) asParameter
		epsilon: (self cppFloat: self epsilon) asParameter
		maxRadius: (self cppFloat: self maxRadius) asParameter! !



!ParticleOrbitPoint methods !
  
value
	" The receiver must perform it's action. "

	self dll
		pOrbitPoint: (self cppFloat: self x) asParameter
		y: (self cppFloat: self y) asParameter
		z: (self cppFloat: self z) asParameter
		magnitude: (self cppFloat: self magnitude) asParameter
		epsilon: (self cppFloat: self epsilon) asParameter
		maxRadius: (self cppFloat: self maxRadius) asParameter! !



!ParticleCopyVertexB methods !
  
copyPosition
    "Returns the instance var of the receiver
     for external manipulation"

	copyPosition isNil ifTrue: [ self initializeCopyPosition ].
    ^copyPosition!

copyPosition: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    copyPosition := anObject!
   
copyVelocity
    "Returns the instance var of the receiver
     for external manipulation"

	copyVelocity isNil ifTrue: [ self initializeCopyVelocity ].
    ^copyVelocity!

copyVelocity: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    copyVelocity := anObject!
   
initializeCopyPosition
	" Private - Initializes the receiver's copyPosition"

	copyPosition := true.!

initializeCopyVelocity
	" Private - Initializes the receiver's copyVelocity"

	copyVelocity := false.!
   
value
	" The receiver must perform it's action. "

	self dll
		pCopyVertexB: self copyPosition asParameter
		copyVel: self copyVelocity asParameter! !



!ParticleKillOld class methods !
 
value: value
	"Returns an instance of the receiver."

	^self new ageLimit: value;yourself! !



!ParticleKillOld methods !
   
ageLimit
    "Returns the instance var of the receiver
     for external manipulation"
    ^ageLimit!

ageLimit: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    ageLimit := anObject!
   
initializeKillLessThan
	" Private - Initializes the receiver's killLessThan"

	killLessThan := false.!
   
killLessThan
    "Returns the instance var of the receiver
     for external manipulation"

	killLessThan isNil ifTrue: [ self initializeKillLessThan ].
    ^killLessThan!

killLessThan: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    killLessThan := anObject!
   
value
	" The receiver must perform it's action. "

	self dll
		pKillOld: (self cppFloat: self ageLimit) asParameter
		killLessThan: self killLessThan asParameter! !



!ParticleKillYoung methods !
   
initializeKillLessThan
	" Private - Initializes the receiver's killLessThan"

	killLessThan := true.! !



!ParticleMove methods !
   
value
	" The receiver must perform it's action. "

	self dll pMove.! !



!ParticleRestore methods !
 
timeLeft
    "Returns the instance var of the receiver
     for external manipulation"
    ^timeLeft!

timeLeft: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    timeLeft := anObject!
   
value
	" The receiver must perform it's action. "

	self dll
		pRestore: (self cppFloat: self timeLeft) asParameter! !



!ParticleSpeedLimit methods !
 
maxSpeed
    "Returns the instance var of the receiver
     for external manipulation"
    ^maxSpeed!

maxSpeed: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    maxSpeed := anObject!
   
minSpeed
    "Returns the instance var of the receiver
     for external manipulation"
    ^minSpeed!

minSpeed: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    minSpeed := anObject!
   
value
	" The receiver must perform it's action. "

	self dll
		pSpeedLimit: (self cppFloat: self minSpeed) asParameter
		max: (self cppFloat: self maxSpeed) asParameter! !



!ParticleTargetColor methods !
  
alpha
    "Returns the instance var of the receiver
     for external manipulation"
    ^alpha!
  
alpha: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    alpha := anObject!
 
blue
    "Returns the instance var of the receiver
     for external manipulation"
    ^blue!

blue: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    blue := anObject!
   
green
    "Returns the instance var of the receiver
     for external manipulation"
    ^green!
  
green: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    green := anObject!
 
red
    "Returns the instance var of the receiver
     for external manipulation"
    ^red!
  
red: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    red := anObject!
 
scale
    "Returns the instance var of the receiver
     for external manipulation"
    ^scale!
  
scale: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    scale := anObject!
 
value
	" The receiver must perform it's action. "

	self dll
		pTargetColor: (self cppFloat: self red) asParameter
		y: (self cppFloat: self green) asParameter
		z: (self cppFloat: self blue) asParameter
		alpha: (self cppFloat: self alpha) asParameter
		scale: (self cppFloat: self scale) asParameter! !



!PointParticleAction class methods !

origin: origin
    "Returns an instance of the receiver."

   ^self new
		origin: origin;
		yourself!
  
x: aX
    "Returns an instance of the receiver."

   ^self new
		x: aX;
		yourself!

x: aX y: aY z: aZ
    "Returns an instance of the receiver."

   ^self new
		x: aX y: aY z: aZ;
		yourself!

y: aY
    "Returns an instance of the receiver."

   ^self new
		y: aY;
		yourself!

z: aZ
    "Returns an instance of the receiver."

   ^self new
		z: aZ;
		yourself! !



!PointParticleAction methods !

initializeX
	" Private - Initializes the receiver's x"

	x := 0.!

initializeY
	" Private - Initializes the receiver's y"

	y := 0.!

initializeZ
	" Private - Initializes the receiver's z"

	z := 0.!

origin: origin
	"Sets the receiver origin."

	self
		x: (origin at: 1);
		y: (origin at: 2);
		z: (origin at: 3)!
 
x
    "Returns the instance var of the receiver
     for external manipulation"

	x isNil ifTrue: [ self initializeX ].
    ^x!

x: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    x := anObject!
 
x: aX y: aY z: aZ
    "Sets the receiver's coordinates."

    self x: aX; y: aY; z: aZ!
  
y
    "Returns the instance var of the receiver
     for external manipulation"

	y isNil ifTrue: [ self initializeY ].
    ^y!

y: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    y := anObject!
 
z
    "Returns the instance var of the receiver
     for external manipulation"

	z isNil ifTrue: [ self initializeZ ].
    ^z!

z: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    z := anObject! !



!ParticleDamping class methods !
   
value: value
	"Returns an instance of the receiver."

	^self x: value y: value z: value! !



!ParticleDamping methods !
 
high
    "Returns the instance var of the receiver
     for external manipulation"

	high isNil ifTrue: [ self initializeHigh ].
    ^high!

high: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    high := anObject!
   
initializeHigh
	" Private - Initializes the receiver's high"

	high := self maxFloat.!
   
initializeLow
	" Private - Initializes the receiver's low"

	low := 0.!
  
low
    "Returns the instance var of the receiver
     for external manipulation"

	low isNil ifTrue: [ self initializeLow ].
    ^low!

low: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    low := anObject!
 
value
	" The receiver must perform it's action. "

	self dll
		pDamping: (self cppFloat: self x) asParameter
		y: (self cppFloat: self y) asParameter
		z: (self cppFloat: self z) asParameter
		vlow: (self cppFloat: self low) asParameter
		vhigh: (self cppFloat: self high) asParameter! !



!ParticleExplosion methods !
 
age
    "Returns the instance var of the receiver
     for external manipulation"

	age isNil ifTrue: [ self initializeAge ].
    ^age!

age: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    age := anObject!
 
epsilon
    "Returns the instance var of the receiver
     for external manipulation"

	epsilon isNil ifTrue: [ self initializeEpsilon ].
    ^epsilon!

epsilon: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    epsilon := anObject!
 
initializeAge
	" Private - Initializes the receiver's age"

	age := 0.!
  
initializeEpsilon
	" Private - Initializes the receiver's epsilon"

	epsilon := self epsilon.!
   
magnitude
    "Returns the instance var of the receiver
     for external manipulation"
    ^magnitude!
  
magnitude: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    magnitude := anObject!
 
stdev
    "Returns the instance var of the receiver
     for external manipulation"
    ^stdev!
  
stdev: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    stdev := anObject!
 
value
	" The receiver must perform it's action. "

	self dll
		pExplosion: (self cppFloat: self x) asParameter
		centerY: (self cppFloat: self y) asParameter
		centerZ: (self cppFloat: self z) asParameter
		velocity: (self cppFloat: self velocity) asParameter
		magnitude: (self cppFloat: self magnitude) asParameter
		stdev: (self cppFloat: self stdev) asParameter
		epsilon: (self cppFloat: self epsilon) asParameter
		age: (self cppFloat: self age) asParameter!
 
velocity
    "Returns the instance var of the receiver
     for external manipulation"
    ^velocity!

velocity: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    velocity := anObject! !



!ParticleGravity methods !
   
value
	" The receiver must perform it's action. "

	self dll
		pGravity: (self cppFloat: self x) asParameter
		y: (self cppFloat: self y) asParameter
		z: (self cppFloat: self z) asParameter! !



!ParticleJet methods !
   
epsilon
    "Returns the instance var of the receiver
     for external manipulation"

    epsilon isNil ifTrue: [ self initializeEpsilon ].
    ^epsilon!
 
epsilon: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    epsilon := anObject!
 
initializeEpsilon
    " Private - Initializes the receiver's epsilon"

    epsilon := super epsilon.!

initializeMagnitude
    " Private - Initializes the receiver's magnitude"

    magnitude := 1.!
  
initializeMaxRadius
    " Private - Initializes the receiver's maxRadius"

    maxRadius := self maxFloat.!
  
magnitude
    "Returns the instance var of the receiver
     for external manipulation"

    magnitude isNil ifTrue: [ self initializeMagnitude ].
    ^magnitude!
 
magnitude: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    magnitude := anObject!
 
maxRadius
    "Returns the instance var of the receiver
     for external manipulation"

    maxRadius isNil ifTrue: [ self initializeMaxRadius ].
    ^maxRadius!
 
maxRadius: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    maxRadius := anObject!
 
value
	" The receiver must perform it's action. "

	self dll
		pJet: (self cppFloat: self x) asParameter
		centerY: (self cppFloat: self y) asParameter
		centerZ: (self cppFloat: self z) asParameter
		magnitude: (self cppFloat: self magnitude) asParameter
		epsilon: (self cppFloat: self epsilon) asParameter
		maxRadius: (self cppFloat: self maxRadius) asParameter! !



!ParticleVortex methods !
  
axisX
    "Returns the instance var of the receiver
     for external manipulation"
    ^axisX!
  
axisX: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    axisX := anObject!
 
axisY
    "Returns the instance var of the receiver
     for external manipulation"
    ^axisY!
  
axisY: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    axisY := anObject!
 
axisZ
    "Returns the instance var of the receiver
     for external manipulation"
    ^axisZ!
  
axisZ: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    axisZ := anObject!
 
value
	" The receiver must perform it's action. "

	self dll
		pVortex: (self cppFloat: self x) asParameter
		y: (self cppFloat: self y) asParameter
		z: (self cppFloat: self z) asParameter
		axisX: (self cppFloat: self axisX) asParameter
		axisY: (self cppFloat: self axisY) asParameter
		axisZ: (self cppFloat: self axisZ) asParameter
		magnitude: (self cppFloat: self magnitude) asParameter
		epsilon: (self cppFloat: self epsilon) asParameter
		maxRadius: (self cppFloat: self maxRadius) asParameter! !



!ParticleTargetSize methods !
 
initializeScaleX
	" Private - Initializes the receiver's scaleX"

	scaleX := 0.!
 
initializeScaleY
	" Private - Initializes the receiver's scaleY"

	scaleY := 0.!
 
initializeScaleZ
	" Private - Initializes the receiver's scaleZ"

	scaleZ := 0.!
 
scaleX
    "Returns the instance var of the receiver
     for external manipulation"

	scaleX isNil ifTrue: [ self initializeScaleX ].
    ^scaleX!

scaleX: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    scaleX := anObject!
   
scaleY
    "Returns the instance var of the receiver
     for external manipulation"

	scaleY isNil ifTrue: [ self initializeScaleY ].
    ^scaleY!

scaleY: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    scaleY := anObject!
   
scaleZ
    "Returns the instance var of the receiver
     for external manipulation"

	scaleZ isNil ifTrue: [ self initializeScaleZ ].
    ^scaleZ!

scaleZ: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    scaleZ := anObject!
   
value
	" The receiver must perform it's action. "

	self dll
		pTargetSize: (self cppFloat: self x) asParameter
		y: (self cppFloat: self y) asParameter
		z: (self cppFloat: self z) asParameter
		scaleX: (self cppFloat: self scaleX) asParameter
		scaleY: (self cppFloat: self scaleY) asParameter
		scaleZ: (self cppFloat: self scaleZ) asParameter! !



!ParticleTargetVelocity methods !
 
scale
    "Returns the instance var of the receiver
     for external manipulation"
    ^scale!
  
scale: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    scale := anObject!
 
value
	" The receiver must perform it's action. "

	self dll
		pTargetVelocity: (self cppFloat: self x) asParameter
		y: (self cppFloat: self y) asParameter
		z: (self cppFloat: self z) asParameter
		scale: (self cppFloat: self scale) asParameter! !



!ParticleVertex methods !
   
value
	" The receiver must perform it's action. "

	self dll
		pVertex: (self cppFloat: self x) asParameter
		y: (self cppFloat: self y) asParameter
		z: (self cppFloat: self z) asParameter! !



!DomainStateSetting class methods !
   
forDomain: aDomain
	"Returns an instance of the receiver on aDomain."

	^self new
		domain: aDomain;
		yourself!
   
on: aDomain
	"Returns an instance of the receiver actin on aDomain."

	^self forDomain: aDomain! !



!DomainStateSetting methods !
  
domain
    "Returns the instance var of the receiver
     for external manipulation"

	domain isNil ifTrue: [ self initializeDomain ].
    ^domain!

domain: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    domain := anObject!
   
domainClass
	" Private - Returns the class used to create the receiver's domain.
	This message must be reimplemented by classes that support default initialization of domains.
	If this method has not been reimplemented, you must set the domain to use this action."

	^self invalidMessage!
   
initializeDomain
	" Private - Initializes the receiver's domain"

	domain := self domainClass new.! !



!DomainColor methods !
  
alpha
    "Returns the instance var of the receiver
     for external manipulation"

	alpha isNil ifTrue: [ self initializeAlpha ].
    ^alpha!

alpha: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    alpha := anObject!
 
initializeAlpha
	" Private - Initializes the receiver's alpha"

	alpha := 1.!

value
	" The receiver must perform it's action. "

	self dll
		pColorD: (self cppFloat: self alpha) asParameter
		dtype: self domain type asParameter
		a0: (self cppFloat: self domain a0) asParameter
		a1: (self cppFloat: self domain a1) asParameter
		a2: (self cppFloat: self domain a2) asParameter
		a3: (self cppFloat: self domain a3) asParameter
		a4: (self cppFloat: self domain a4) asParameter
		a5: (self cppFloat: self domain a5) asParameter
		a6: (self cppFloat: self domain a6) asParameter
		a7: (self cppFloat: self domain a7) asParameter
		a8: (self cppFloat: self domain a8) asParameter! !



!DomainSize methods !
   
value
	" The receiver must perform it's action. "

	self dll
		pSizeD: self domain type asParameter
		a0: (self cppFloat: self domain a0) asParameter
		a1: (self cppFloat: self domain a1) asParameter
		a2: (self cppFloat: self domain a2) asParameter
		a3: (self cppFloat: self domain a3) asParameter
		a4: (self cppFloat: self domain a4) asParameter
		a5: (self cppFloat: self domain a5) asParameter
		a6: (self cppFloat: self domain a6) asParameter
		a7: (self cppFloat: self domain a7) asParameter
		a8: (self cppFloat: self domain a8) asParameter! !



!DomainVelocity methods !
  
value
	" The receiver must perform it's action. "

	self dll
		pVelocityD: self domain type asParameter
		a0: (self cppFloat: self domain a0) asParameter
		a1: (self cppFloat: self domain a1) asParameter
		a2: (self cppFloat: self domain a2) asParameter
		a3: (self cppFloat: self domain a3) asParameter
		a4: (self cppFloat: self domain a4) asParameter
		a5: (self cppFloat: self domain a5) asParameter
		a6: (self cppFloat: self domain a6) asParameter
		a7: (self cppFloat: self domain a7) asParameter
		a8: (self cppFloat: self domain a8) asParameter! !



!DomainVertexB methods !
   
value
	" The receiver must perform it's action. "

	self dll
		pVertexBD: self domain type asParameter
		a0: (self cppFloat: self domain a0) asParameter
		a1: (self cppFloat: self domain a1) asParameter
		a2: (self cppFloat: self domain a2) asParameter
		a3: (self cppFloat: self domain a3) asParameter
		a4: (self cppFloat: self domain a4) asParameter
		a5: (self cppFloat: self domain a5) asParameter
		a6: (self cppFloat: self domain a6) asParameter
		a7: (self cppFloat: self domain a7) asParameter
		a8: (self cppFloat: self domain a8) asParameter! !



!ParticleColor class methods !
  
red: red green: green blue: blue alpha: alpha
    "Returns an instance of the receiver."

	^self new
		red: red green: green blue: blue alpha: alpha;
		yourself!
  
rgb: values
    "Returns an instance of the receiver."

	^self new
		red: (values at: 1)
		green: (values at: 2)
		blue: (values at: 3)
		alpha: 1;
		yourself!
 
value: values
    "Returns an instance of the receiver."

	^self rgb: values! !



!ParticleColor methods !
  
alpha
    "Returns the instance var of the receiver
     for external manipulation"

	alpha isNil ifTrue: [ self initializeAlpha ].
    ^alpha!

alpha: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    alpha := anObject!
 
blue
    "Returns the instance var of the receiver
     for external manipulation"
    ^blue!

blue: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    blue := anObject!
   
green
    "Returns the instance var of the receiver
     for external manipulation"
    ^green!
  
green: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    green := anObject!
 
initializeAlpha
	" Private - Initializes the receiver's alpha"

	alpha := 1.!

red
    "Returns the instance var of the receiver
     for external manipulation"
    ^red!
  
red: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    red := anObject!
 
red: r green: g blue: b
    "Sets the instance var of the receiver."

	red := r.
	green := g.
	blue := b.!
 
red: r green: g blue: b alpha: aNumber
    "Sets the instance var of the receiver."

	self
		red: r green: g blue: b;
		alpha: aNumber!

value
	" The receiver must perform it's action. "

	self dll
		pColor: (self cppFloat: self red) asParameter
		green: (self cppFloat: self green) asParameter
		blue: (self cppFloat: self blue) asParameter
		alpha: (self cppFloat: self alpha) asParameter.! !



!ParticleCurrentGroup methods !
 
group
    "Returns the instance var of the receiver
     for external manipulation"
    ^group!
  
group: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    group := anObject!
 
value
	" The receiver must perform it's action. "

	self group makeCurrent.! !



!ParticleStartingAge class methods !
   
value: value
	"Returns an instance of the receiver."

	^self new age: 0; yourself! !



!ParticleStartingAge methods !
   
age
    "Returns the instance var of the receiver
     for external manipulation"
    ^age!
  
age: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    age := anObject!
 
initializeStdev
	" Private - Initializes the receiver's stdev"

	stdev := 0.!

stdev
    "Returns the instance var of the receiver
     for external manipulation"

	stdev isNil ifTrue: [ self initializeStdev ].
    ^stdev!

stdev: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    stdev := anObject!
 
value
	" The receiver must perform it's action. "

	self dll
		pStartingAge: (self cppFloat: self age) asParameter
		stdev: (self cppFloat: self stdev) asParameter.! !



!ParticleTimeStep methods !
 
dt
    "Returns the instance var of the receiver
     for external manipulation"
    ^dt!

dt: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    dt := anObject!
   
value
	" The receiver must perform it's action. "

	self dll pTimeStep: (self cppFloat: self dt) asParameter.! !



!ParticleVertexBTracks methods !
 
doCopy
    "Returns the instance var of the receiver
     for external manipulation"

	doCopy isNil ifTrue: [ self initializeDoCopy ].
    ^doCopy!

doCopy: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    doCopy := anObject!
   
initializeDoCopy
	" Private - Initializes the receiver's doCopy"

	doCopy := true.!
  
value
	" The receiver must perform it's action. "

	self dll pVertexBTracks: self doCopy asParameter.! !



!PointStateSetting class methods !
   
x: aX
    "Returns an instance of the receiver."

   ^self new
		x: aX;
		yourself!

x: aX y: aY z: aZ
    "Returns an instance of the receiver."

   ^self new
		x: aX y: aY z: aZ;
		yourself! !



!PointStateSetting methods !
  
initializeX
	" Private - Initializes the receiver's x"

	x := 0.!

initializeY
	" Private - Initializes the receiver's y"

	y := 0.!

initializeZ
	" Private - Initializes the receiver's z"

	z := 0.!

x
    "Returns the instance var of the receiver
     for external manipulation"

	x isNil ifTrue: [ self initializeX ].
    ^x!

x: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    x := anObject!
 
x: aX y: aY z: aZ
    "Sets the receiver's coordinates."

    self x: aX; y: aY; z: aZ!
  
y
    "Returns the instance var of the receiver
     for external manipulation"

	y isNil ifTrue: [ self initializeY ].
    ^y!

y: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    y := anObject!
 
z
    "Returns the instance var of the receiver
     for external manipulation"

	z isNil ifTrue: [ self initializeZ ].
    ^z!

z: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    z := anObject! !



!ParticleSize methods !

initializeY
	" Private - Initializes the receiver's y"

	y := 1.!

initializeZ
	" Private - Initializes the receiver's z"

	z := 1.!

value
	" The receiver must perform it's action. "

	self dll
		pSize: (self cppFloat: self x) asParameter
		y: (self cppFloat: self y) asParameter
		z: (self cppFloat: self z) asParameter.! !



!ParticleVelocity methods !

value
	" The receiver must perform it's action. "

	self dll
		pVelocity: (self cppFloat: self x) asParameter
		y: (self cppFloat: self y) asParameter
		z: (self cppFloat: self z) asParameter.! !



!ParticleVertexB methods !
 
value
	" The receiver must perform it's action. "

	self dll
		pVertexB: (self cppFloat: self x) asParameter
		y: (self cppFloat: self y) asParameter
		z: (self cppFloat: self z) asParameter.! !



!ParticleDomain class methods !
 
from: aCollection
	"Returns an instance of the receiver set with aColection of values."

	^self values: aCollection!
 
new
	"Answer a new instance of the receiver."

	^super new initialize!
   
values: aCollection
	"Returns an instance of the receiver set with aColection of values."

	^self new
		values: aCollection;
		yourself! !



!ParticleDomain methods !

a0
    "Returns the instance var of the receiver
     for external manipulation"
    ^a0!

a0: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    a0 := anObject!
   
a1
    "Returns the instance var of the receiver
     for external manipulation"
    ^a1!

a1: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    a1 := anObject!
   
a2
    "Returns the instance var of the receiver
     for external manipulation"
    ^a2!

a2: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    a2 := anObject!
   
a3
    "Returns the instance var of the receiver
     for external manipulation"
    ^a3!

a3: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    a3 := anObject!
   
a4
    "Returns the instance var of the receiver
     for external manipulation"
    ^a4!

a4: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    a4 := anObject!
   
a5
    "Returns the instance var of the receiver
     for external manipulation"
    ^a5!

a5: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    a5 := anObject!
   
a6
    "Returns the instance var of the receiver
     for external manipulation"
    ^a6!

a6: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    a6 := anObject!
   
a7
    "Returns the instance var of the receiver
     for external manipulation"
    ^a7!

a7: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    a7 := anObject!
   
a8
    "Returns the instance var of the receiver
     for external manipulation"
    ^a8!

a8: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    a8 := anObject!
   
initialize
	" Private - Initializes the receiver. "

	a0 := a1 := a2 := a3 := a4 := a5 := a6 := a7 := a8 := 0.!
  
type
	"Returns the type constant of the receiver."

	^self implementedBySubclass!

typeBlob
	" Private - Returns the type constant."

	^PDBlob!
 
typeBox
	" Private - Returns the type constant."

	^PDBox!
   
typeCone
	" Private - Returns the type constant."

	^PDCone!
 
typeCylinder
	" Private - Returns the type constant."

	^PDCylinder!
 
typeDisc
	" Private - Returns the type constant."

	^PDDisc!
 
typeLine
	" Private - Returns the type constant."

	^PDLine!
 
typePlane
	" Private - Returns the type constant."

	^PDPlane!
   
typePoint
	" Private - Returns the type constant."

	^PDPoint!
   
typeRectangle
	" Private - Returns the type constant."

	^PDRectangle!
   
typeSphere
	" Private - Returns the type constant."

	^PDSphere!
 
typeTriangle
	" Private - Returns the type constant."

	^PDTriangle!
 
v1
	"Returns the first vector coordinates of the receiver."

	^Array with: a0 with: a1 with: a2!
 
v1: anArray
	"Sets the first vector coordinates of the receiver."

	a0 := anArray at: 1.
	a1 := anArray at: 2. 
	a2 :=  anArray at: 3.!

v2
	"Returns the second vector coordinates of the receiver."

	^Array with: a3 with: a4 with: a5!

v2: anArray
	"Sets the second vector coordinates of the receiver."

	a3 := anArray at: 1.
	a4 := anArray at: 2. 
	a5 :=  anArray at: 3.!
   
v3
	"Returns the third vector coordinates of the receiver."

	^Array with: a6 with: a7 with: a8!
 
v3: anArray
	"Sets the third vector coordinates of the receiver."

	a6 := anArray at: 1.
	a7 := anArray at: 2. 
	a8 :=  anArray at: 3.!

values: aCollection
	"Sets the receiver with aCollection of values."

	1 to: aCollection size do: [:index|
		self instVarAt: index put: (aCollection at: index)
	]! !



!CenteredDomain class methods !
   
origin: anArray
	"Returns an instance of the receiver."

	^self new origin: anArray; yourself! !



!CenteredDomain methods !

origin
	"Returns the origin coordinates of the receiver."

	^self v1!

origin: anArray
	"Sets the origin coordinates of the receiver."

	^self v1: anArray! !



!BoxDomain class methods !
 
origin: origin corner: corner
	"Returns an instance of the receiver."

	^(self origin: origin)
		corner: corner;
		yourself! !



!BoxDomain methods !
 
corner
	"Returns the corner coordinates of the receiver."

	^self v2!

corner: anArray
	"Sets the corner coordinates of the receiver."

	^self v2: anArray!
 
type
	"Returns the type constant of the receiver."

	^self typeBox! !



!PlaneDomain class methods !

origin: origin normal: normal
	"Returns an instance of the receiver."

	^self new
		origin: origin;
		normal: normal;
		yourself! !



!PlaneDomain methods !
 
normal
	"Returns the normal coordinates of the receiver."

	^self v2!

normal: anArray
	"Sets the normal coordinates of the receiver."

	^self v2: anArray!
 
type
	"Returns the type constant of the receiver."

	^self typePlane! !



!DiscDomain methods !
 
innerRadius
	"Returns the innerRadius of the receiver."

	^self a7!
  
innerRadius: aNumber
	"Set the innerRadius of the receiver."

	^self a7: aNumber!

outerRadius
	"Returns the outerRadius of the receiver."

	^self a6!
  
outerRadius: aNumber
	"Set the outerRadius of the receiver."

	^self a6: aNumber!

radius: aNumber
	"Set the radius of the receiver."

	^self outerRadius: aNumber!
 
type
	"Returns the type constant of the receiver."

	^self typeDisc! !



!RectangleDomain class methods !
   
origin: anArray u: uArray v: vArray
	"Returns an instance of the receiver."

	^(self origin: anArray)
		u: uArray;
		v: vArray;
		yourself! !



!RectangleDomain methods !
   
type
	"Returns the type constant of the receiver."

	^self typeRectangle!

u
	"Returns the u coordinates of the receiver."

	^self v2!
  
u: anArray
	"Sets the u coordinates of the receiver."

	^self v2: anArray!
   
v
	"Returns the v coordinates of the receiver."

	^self v3!
  
v: anArray
	"Sets the v coordinates of the receiver."

	^self v3: anArray! !



!SphereDomain methods !
  
innerRadius
	"Returns the innerRadius of the receiver."

	^self a4!
  
innerRadius: aNumber
	"Set the innerRadius of the receiver."

	^self a4: aNumber!

isSphere
	"Returns true if the receiver is an sphere"

	^self innerRadius = 0
	!

outerRadius
	"Returns the outerRadius of the receiver."

	^self a3!
  
outerRadius: aNumber
	"Set the outerRadius of the receiver."

	^self a3: aNumber!

radius: aNumber
	"Set the radius of the receiver."

	^self outerRadius: aNumber!
 
type
	"Returns the type constant of the receiver."

	^self typeSphere! !



!ConeDomain methods !

apex
	"Returns the apex coordinates of the receiver."

	^self v1!

apex: anArray
	"Sets the apex coordinates of the receiver."

	^self v1: anArray!
 
baseCenter
	"Returns the baseCenter coordinates of the receiver."

	^self v2!

baseCenter: anArray
	"Sets the baseCenter coordinates of the receiver."

	^self v2: anArray!
 
baseRadius
	"Returns the baseRadius of the receiver."

	^self a6!

baseRadius: aNumber
	"Set the baseRadius of the receiver."

	^self a6: aNumber!
  
type
	"Returns the type constant of the receiver."

	^self typeCone!
 
upperRadius
	"Returns the upperRadius of the receiver."

	^self a7!
  
upperRadius: aNumber
	"Set the upperRadius of the receiver."

	^self a7: aNumber! !



!LineDomain class methods !
   
from: source to: destination
	"Returns an instance of the receiver."

	^self new from: source to: destination! !



!LineDomain methods !

destination
	"Returns the destination coordinates of the receiver."

	^self v2!
  
destination: anArray
	"Sets the destination coordinates of the receiver."

	^self v2: anArray!
   
from: source to: destination
	"Sets the source&destination coordinates of the receiver."

	self
		source: source;
		destination: destination!
  
source
	"Returns the source coordinates of the receiver."

	^self v1!

source: anArray
	"Sets the source coordinates of the receiver."

	^self v1: anArray!
 
type
	"Returns the type constant of the receiver."

	^self typeLine! !



!CylinderDomain methods !
  
innerRadius
	"Returns the innerRadius of the receiver."

	^self a7!
  
innerRadius: aNumber
	"Set the innerRadius of the receiver."

	^self a7: aNumber!

outerRadius
	"Returns the outerRadius of the receiver."

	^self a6!
  
outerRadius: aNumber
	"Set the outerRadius of the receiver."

	^self a6: aNumber!

radius: aNumber
	"Set the radius of the receiver."

	^self outerRadius: aNumber!
 
type
	"Returns the type constant of the receiver."

	^self typeCylinder! !



!PointDomain class methods !
   
origin: origin
	"Returns an instance of the receiver."

	^self new origin: origin; yourself!
 
x: x y: y z: z
	"Returns an instance of the receiver."

	^self new x: x; y: y; z: z; yourself! !



!PointDomain methods !
   
origin: anArray
	"Sets the origin coordinates of the receiver."

	^self v1: anArray!
 
type
	"Returns the type constant of the receiver."

	^self typePoint!

x
	"Returns the x of the receiver."

	^self a0!
  
x: aNumber
	"Sets the x of the receiver."

	^self a0: aNumber!
   
y
	"Returns the y of the receiver."

	^self a1!
  
y: aNumber
	"Sets the y of the receiver."

	^self a1: aNumber!
   
z
	"Returns the z of the receiver."

	^self a2!
  
z: aNumber
	"Sets the z of the receiver."

	^self a2: aNumber! !



!BlobDomain class methods !
  
origin: origin stdev: stdev
	"Returns an instance of the receiver."

	^(self origin: origin)
		stdev: stdev;
		yourself! !



!BlobDomain methods !

stdev
	"Returns stdev of the receiver."

	^self a3!
  
stdev: aNumber
	"Set stdev of the receiver."

	^self a3: aNumber!

type
	"Returns the type constant of the receiver."

	^self typeBlob! !



!TriangleDomain methods !
  
type
	"Returns the type constant of the receiver."

	^self typeTriangle!
 
vertex1
	"Returns the first vertex of the receiver."

	^self v1!
 
vertex1: values
	"Set the first vertex of the receiver."

	^self v1: values!
 
vertex2
	"Returns the second vertex of the receiver."

	^self v2!

vertex2: values
	"Set the second vertex of the receiver."

	^self v2: values!

vertex3
	"Returns the third vertex of the receiver."

	^self v3!
 
vertex3: values
	"Set the third vertex of the receiver."

	^self v3: values!
 
vertices
	"Returns the vertices of the receiver."

	^Array
		with: self vertex1
		with: self vertex2
		with: self vertex3!

vertices: aCollection
	"Sets the vertices of the receiver."

	self vertex1: (aCollection at: 1).
	self vertex2: (aCollection at: 2).
	self vertex3: (aCollection at: 3).! !



!ParticleExample class methods !

constructEventsTriggered
	"Private - answer the set of events that instances of the receiver can trigger."

    ^super constructEventsTriggered
		add: #needsToDrawParticles:;
		yourself! !



!ParticleExample methods !
 
aboutToRender
	" Private - Perform preRendering action. "

	self preRenderAction value.!
 
bitmapName
	" Private - Returns the receiver's bitmap name"

	^'spoty'!
  
bitmapScale
	" Private - Returns the scale used to render particle bitmap. "

	^1!
   
colorScale
	" Private - Returns the scale used to scale colors.
	This method can be refined by the subclasses to scale colores on render."

	^1 "colors are in correct scale (0...255)"!

defaultBitmap
	" Private - Returns the default bitmap. "

	defaultBitmap isNil ifTrue: [ self initializeDefaultBitmap ].
	^defaultBitmap!
   
defaultBitmapIn: aWorld
	" Private - Returns the default bitmap in aWorld. "

	(defaultBitmap notNil
	and: [ aWorld hasBitmap: defaultBitmap ])
	ifTrue: [ ^defaultBitmap ].

	"defaultBitmap release."
	self initializeDefaultBitmap.
	defaultBitmap notNil ifTrue: [
		aWorld addBitmap: defaultBitmap
	].
	^defaultBitmap!
   
draw
	" Private - Callback Draw the Particles. "

	self group render.!
   
drawParticles: particles
	" Private - Particles in the receiver's group has changed,
	 and need to be rendered.
	Warning: This message is sent in a critical(callback) state
		and it is recomended not to be a time consuming behavior."

	self triggerEvent: #needsToDrawParticles: with: particles.!
   
drawScale
	" Private - Returns the scale used to render.
	The default action is to return nil (unscaled).
	This message can be reimplemented (returning the scale factor [aNumber]) "

	^nil!
  
edit
	"Edit the receiver."

	^self editorClass new openOn: self!
 
editFullScreen
	"Edit the receiver in FullScreen mode."

	^self editorClass new openFullScreenOn: self!
  
editorClass
	"Returns the calss used to edit the receiver."

	| aName |
	self class withAllSuperclasses do: [:aClass|
		aName := (aClass name ,'Editor') asSymbol.
		(Smalltalk includesKey: aName)
		ifTrue: [ ^Smalltalk at: aName ].
	].
	self error: 'Editor not found.'!
  
group
    "Returns the instance var of the receiver
     for external manipulation"

	group isNil ifTrue: [ self initializeGroup ].
    ^group!

group: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    group := anObject!
 
initializeDefaultBitmap
	" Private - Initializes the receiver's defaultBitmap"

	defaultBitmap := GenesisBitmap
		fromPathName: self bitmapName,'.bmp'
		alpha: 'A_',self bitmapName,'.bmp'.
	defaultBitmap pixelFormat: '16Bit4444ARGB'.!

initializeGroup
	" Private - Initializes the receiver's group"

	group := ParticleGroup new: self maxParticleCount.
	group
		when: #aboutToRender send: #aboutToRender to: self;
		when: #particlesChanged: send: #drawParticles: to: self.!
  
initializePreRenderAction
	" Private - Initialize the receiver's pre-render action.
	The normal behavior is to initialize with an empty action.
	This method can be refined by the subclasses."

	preRenderAction := ParticleAction empty.!

maxParticleCount
	" Private - Returns the maximum number of particles supported by the example. "

	^ParticleGroup defaultMaxParticle!
   
maxParticles
	" Private - Returns the maximum number of particles set in the example. "

	^self maxParticleCount!

preRenderAction
    "Returns the instance var of the receiver
     for external manipulation"

	preRenderAction isNil ifTrue: [ self initializePreRenderAction ].
    ^preRenderAction!

preRenderAction: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    preRenderAction := anObject!
 
release
	" Release the receiver. "

	group release.
	^super release! !



!ParticleNormalizedExample methods !
  
bitmapScale
	" Private - Returns the scale used to render particle bitmap. "

	^1/5!
 
colorScale
	" Private - Returns the scale used to scale colors."

	^1!
   
drawScale
	" Private - Returns the scale used to render."

	^10!
 
maxParticleCount
	" Private - Returns the maximum number of particles supported by the example. "

	^300
	"Prompter
		prompt: 'Maximum particle count ?'
		defaultExpression: 150 printString
	"! !



!Balloons methods !
   
initializePreRenderAction
	" Private - Initialize the receiver's pre-render action. "

	preRenderAction :=
	(ParticleStartingAge new age: 0; stdev: 5; yourself)
	, (ParticleVelocity x: 0 y: 0 z: 0).

	#(
		(255 0 0)		(255 255 0)
		(0 255 0)		(0 255 255)
		(0 0 255)		(255 0 255)
	) do: [:color|
		preRenderAction := preRenderAction
			, (ParticleColor value: color)
			, ((DomainSource
				origin: #(-0.5 -0.5 -1.5)
				corner: #(0.5 0.5 -0.5) )
				rate: self maxParticles / 6)
	].

	preRenderAction := preRenderAction
	, (ParticleGravity x: 0.0005 y: 0.005 z: 0.0005)
	, (ParticleDamping x: 0.9 y: 0.67 z: 0.9)
	, (DomainRandomAcceleration origin: #(-0.05 -0.05 -0.05) corner: #(0.5 0.5 0.5 ))
	, (ParticleKillOld value: 40"400")
	, ParticleMove new
	, (ParticleStartingAge value: 0)! !



!Explosion methods !
  
age
    "Returns the instance var of the receiver
     for external manipulation"

	age isNil ifTrue: [ self initializeAge ].
    ^age!

explosionAge
	" Private - Returns the explosion age value. "

	^age := 1 / self numSteps + self age!
 
initializeAge
	" Private - Initializes the receiver's age"

	age := 0.!
  
initializeNumSteps
	" Private - Initializes the receiver's numSteps"

	numSteps := 7.!
   
initializePreRenderAction
	" Private - Initialize the receiver's pre-render action. "

	preRenderAction :=
	(DomainVelocity sphereAt: #(0 0 0.01) radius: 0.01 )
	, (DomainColor sphereAt: #(128 180 128) radius: 178 )
	, (ParticleSize x: 1)
	, (ParticleStartingAge value: 0)
	, ParticleCopyVertexB new

	, ((DomainSource sphereAt: #(0 0 0) radius: 3 ) rate: 17)

	, ((ParticleDamping value: 0.999) low: 0.2)
	, ((ParticleOrbitPoint x: 0 y: 0 z: 0) magnitude: 0.02; epsilon:0.1 )
	, ((ParticleExplosion x: 0 y: 0 z: 0)
		velocity: 1; magnitude: 8; stdev: 3; epsilon: 0.1;
		age: [self explosionAge])
	, (DomainSinkOutside sphereAt: #(0 0 0) radius: 20 )
	, ParticleMove new!
   
numSteps
    "Returns the instance var of the receiver
     for external manipulation"

	numSteps isNil ifTrue: [ self initializeNumSteps ].
    ^numSteps! !



!FireFlies methods !
  
initializePreRenderAction
	" Private - Initialize the receiver's pre-render action. "

	preRenderAction :=

	(DomainVelocity pointAt: #(0 0 0))
	", (DomainColor sphereAt: #(128 120 25) radius: 25 )"
	, (DomainColor sphereAt: #(255 255 255) radius: 250 )
	, (ParticleSize x: 1)
	, ParticleCopyVertexB new
	, ((DomainSource blobAt: #(0 0 2) stdev: 2) rate: 2)
	, (DomainRandomAcceleration sphereAt: #(0 0 0) radius: 0.5"0.02")
	, (ParticleKillOld value: 20)
	, ParticleMove new! !



!Fountain methods !
  
bitmapName
	" Private - Returns the receiver's bitmap name"

	^'buble'!
  
initializePreRenderAction
	" Private - Initialize the receiver's pre-render action. "

	preRenderAction :=

	"Set up the state."
	(DomainVelocity on: (
		(CylinderDomain
			from:	#(0 0 0.35)
			to: 		#(0 0 0.37)
		)	outerRadius: 0.021;
			innerRadius: 0.019;
			yourself) )
	, (DomainColor from: #(20 200 255) to: #(255 255 255) )
	, (ParticleSize x: 1.5)
	, ParticleCopyVertexB new

	"Generate particles along a very small line in the nozzel."
	, ((DomainSource
		lineFrom: #(0 0 0.401) to: #(0 0 0.405) )
		rate: 2"particles per evaluation")

	"Apply Gravity."
	, (ParticleGravity z: -0.0098)

	"Bounce particles off a disc of radius 5."
	, ((DomainBounce
		discAt: #(0 0 0) normal: #(0 0 1) radius: 5 )
		friction: -0.05;
		resilience: 0.35;
		cutoff: 0)

	"Kill particles below Z=-3."
	, (DomainSinkOutside planeAt: #(0 0 -3) normal: #(0 0 1) )

	"Move the particles"
	, ParticleMove new! !



!JetExample methods !
 
deltaFactor
	" Private - Returns the receiver's delta factor to used during dj initialization."

	^0.5!
  
djx
    "Returns the instance var of the receiver
     for external manipulation"

	djx isNil ifTrue: [ self initializeDjx ].
    ^djx!

djy
    "Returns the instance var of the receiver
     for external manipulation"

	djy isNil ifTrue: [ self initializeDjy ].
    ^djy!

djz
    "Returns the instance var of the receiver
     for external manipulation"

	djz isNil ifTrue: [ self initializeDjz ].
    ^djz!

initializeDjx
	" Private - Initializes the receiver's djx"

	djx := self randomizer next * self deltaFactor.!

initializeDjy
	" Private - Initializes the receiver's djy"

	djy := self randomizer next * self deltaFactor.!

initializeDjz
	" Private - Initializes the receiver's djz"

	djz := self randomizer next * self deltaFactor.!

initializeJetX
	" Private - Initializes the receiver's jetX"

	jetX := 0.!
   
initializeJetY
	" Private - Initializes the receiver's jetY"

	jetY := 0.!
   
initializeJetZ
	" Private - Initializes the receiver's jetZ"

	jetZ := 0.!
   
initializeRandomizer
	" Private - Initializes the receiver's randomizer"

	randomizer := (Smalltalk at: #GenesisRandomNumberGenerator) new.!
 
jetX
    "Returns the instance var of the receiver
     for external manipulation"

	jetX isNil ifTrue: [ self initializeJetX ].
    ^jetX!

jetY
    "Returns the instance var of the receiver
     for external manipulation"

	jetY isNil ifTrue: [ self initializeJetY ].
    ^jetY!

jetZ
    "Returns the instance var of the receiver
     for external manipulation"

	jetZ isNil ifTrue: [ self initializeJetZ ].
    ^jetZ!

randomizer
    "Returns the instance var of the receiver
     for external manipulation"

	randomizer isNil ifTrue: [ self initializeRandomizer ].
    ^randomizer! !



!JetSpray methods !
   
computeJetValues
	" Private - Compute the coordinates to use for Jet particles."

	jetX := self jetX + self djx.
	jetY := self jetY + self djy.
	(jetX between: -10 and: 10) ifFalse: [
		djx := djx negated.
		djy := self randomizer next * 0.005 + djy.
	].
	(jetY between: -10 and: 10) ifFalse: [
		djy := djy negated.
		djx := self randomizer next * 0.005 + djx.
	].!
  
initializePreRenderAction
	" Private - Initialize the receiver's pre-render action. "

	preRenderAction :=

	(DomainVelocity blobAt: #(0 0 0) stdev: 0.01)
	, (ParticleSize x: 1.5)
	, ParticleCopyVertexB new

	, (DomainColor sphereAt: #(240 102 25) radius: 25 )
	, ((DomainSource
		rectangleAt: #(-1 -1 0.1) u: #(2 0 0) v: #(0 2 0) )
		rate: 1)
	, (DomainColor sphereAt: #(128 102 25) radius: 25 )
	, ((DomainSource
		rectangleAt: #(-10 -10 0.1) u: #(20 0 0) v: #(0 20 0) )
		rate: 5)
	, (ParticleGravity z: -0.0098)

	, (DomainVelocity blobAt: #(0 0 0.05) stdev: 0.01)
	, ((ParticleJet x: [self computeJetValues;jetZ] y: [self jetY] z: [self jetZ])
			magnitude: 3; epsilon: 0.01; maxRadius: 1.5;
			yourself)

	, ((DomainBounce rectangleAt: #(-10 10 0) u: #(20 0 0) v: #(0 20 0) )
		friction: 0; resilience: 0.35; cutoff: 0)
	, (DomainSinkOutside planeAt: #(0 0 -20) normal: #(0 0 1) )
	, ParticleMove new! !



!Snake methods !

computeJetValues
	" Private - Compute the coordinates to use for Jet particles."

	jetX := self jetX + self djx.
	jetY := self jetY + self djy.
	jetZ := self jetZ + self djz.
	(jetX between: -6 and: 6) ifFalse: [ djx := djx negated ].
	(jetY between: -6 and: 6) ifFalse: [ djy := djy negated ].
	(jetZ between: -6 and: 6) ifFalse: [ djz := djz negated ].!
 
deltaFactor
	" Private - Returns the receiver's delta factor to used during dj initialization."

	^0.05!
 
firstTimeAction
	" Private - Returns the action to be evaluated only the first time.
	We currently use aBlock as an action. "

	| firstTime |
	^[	firstTime isNil
		ifTrue: [ self startUpAction value ].
		firstTime := false.
	]!
 
initializePreRenderAction
	" Private - Initialize the receiver's pre-render action. "

	preRenderAction :=
	ParticleAction empty
	, self firstTimeAction
	, ParticleCopyVertexB new
	, (ParticleKillYoung value: -1)
	, (ParticleStartingAge value: -10)
	, (ParticleColor red: 255 green: 0 blue: 0 alpha: 1)
	, (ParticleVertex
			x: [self computeJetValues;jetX ]
			y: [ self jetY ] z: [ self jetZ ])
	, (ParticleStartingAge value: 0)
	, (ParticleFollow new
			magnitude: 0.01;
			epsilon: 1)
	, (ParticleDamping value: 0.9)
	, ParticleMove new!
  
startUpAction
	" Private - Returns the action to be evaluated during startUp. "

	| action |
	action := (ParticleVelocity x: 0 y: 0 z: 0)
		, (ParticleSize x: 1)
		, (ParticleStartingAge value: 0)
		, (ParticleKillOld value: -300)
		, (DomainColor sphereAt: #(250 250 0) radius: 25 ).
	-10 to: 2 by: 0.1 do: [:x|
		action := action, (ParticleVertex x: x y: 0 z: 0)
	].
	^action! !



!Swirl methods !

applyDamping
	" Private - Apply damping to pre-render action. "

	preRenderAction := preRenderAction
	, (ParticleDamping x: 1 y: 0.994 z: 0.994)!
   
applyMotion
    " Private - Apply motions to pre-render action. "

    preRenderAction := preRenderAction
    , ((ParticleOrbitLine x: 0 y: 0 z: 1)
        axis: 1 y: 0 z: 0;
        magnitude: 0.1;
        epsilon: 0.1;
        maxRadius: 99)!

applyMove
	" Private - Apply move to pre-render action. "

	preRenderAction := preRenderAction
	, (DomainSinkOutside sphereAt: #(0 0 0) radius: 15 )
	, ParticleMove new!
  
colorValues
	" Private - Returns the colors used for particles. "

	^Array
		with: [
				self computeJetValues.
				self jetX * 25 + 102 ]
		with: [ self jetY * 25 + 102 ]
		with: [ self jetZ * 25 + 102 ]!
   
computeJetValues
	" Private - Compute the coordinates to use for Jet particles."

	jetX := self jetX + self djx.
	jetY := self jetY + self djy.
	jetZ := self jetZ + self djz.
	(jetX between: -10 and: 10) ifFalse: [ djx := djx negated ].
	(jetY between: -10 and: 10) ifFalse: [ djy := djy negated ].
	(jetZ between: -10 and: 10) ifFalse: [ djz := djz negated ].!
   
deltaFactor
	" Private - Returns the receiver's delta factor to used during dj initialization."

	^0.05!
 
initializeJetX
	" Private - Initializes the receiver's jetX"

	jetX := -4.!
  
initializeJetZ
	" Private - Initializes the receiver's jetZ"

	jetZ := -2.4.!

initializePreRenderAction
	" Private - Initialize the receiver's pre-render action. "

	preRenderAction :=
	(DomainVelocity blobAt: #(0.02 -0.2 0) stdev: 0.015)
	, (ParticleSize x: 1)
	, ParticleCopyVertexB new
	, (ParticleKillOld value: self lifeTime)
	, (DomainColor sphereAt: self colorValues radius: 25 )
	, ((DomainSource pointAt: self jetValues)
		rate: self maxParticles / self lifeTime) .

	self
		applyMotion;
		applyDamping;
		applyMove;
		yourself.!

jetValues
	" Private - Returns the jet values used for particles. "

	^Array
		with: [ self jetX ]
		with: [ self jetY ]
		with: [ self jetZ ]!
   
lifeTime
	" Private - Returns the lifeTime."

	^300! !



!Chaos methods !
   
applyDamping
	" Private - Apply damping to pre-render action. "

	preRenderAction := preRenderAction
		, (ParticleDamping value: 0.994)!

applyMotion
    " Private - Apply motions to pre-render action. "

    preRenderAction := preRenderAction
	, ((ParticleOrbitPoint x: 2 y: 0 z: 3)
		magnitude: 0.5; epsilon: 0.1; maxRadius: 99)
	, ((ParticleOrbitPoint x: -2 y: 0 z: -3)
		magnitude: 0.5; epsilon: 0.1; maxRadius: 99)!
   
applyMove
	" Private - Apply move to pre-render action. "

	preRenderAction := preRenderAction
	" , (DomainSinkOutside sphereAt: #(0 0 0) radius: 15 ) "
	, ParticleMove new!
  
lifeTime
	" Private - Returns the lifeTime."

	^350! !



!Disperse methods !

applyDamping
	" Private - Apply damping to pre-render action. "

	preRenderAction := preRenderAction
	, (ParticleDamping x: 0.9 y: 0.97 z: 0.97)!
   
applyMotion
    " Private - Apply motions to pre-render action. "

    preRenderAction := preRenderAction
	, (DomainRandomAcceleration
			origin: #(-0.17 -0.17 -0.17)
			corner: #(0.17 0.17 0.17))!
 
applyMove
	" Private - Apply move to pre-render action. "

	preRenderAction := preRenderAction
	, ((DomainBounce planeAt: #(0 0 3) normal: #(0 0 1) )
		friction: 0.05; resilience: 0.3; cutoff: 0)
	, ParticleMove new! !



!Vortex methods !
   
applyDamping
	" Private - Apply damping to pre-render action. "!
   
applyMotion
    " Private - Apply motions to pre-render action. "

    preRenderAction := preRenderAction
	, ((ParticleVortex x: [self computeJetValues;jetX] y: [self jetY] z: 1)
			axisX: 0; axisY: 0; axisZ: 1;
			magnitude: 7;
			yourself)!
   
applyMove
	" Private - Apply move to pre-render action. "

	preRenderAction := preRenderAction
	, ParticleMove new! !



!ParticleAtom methods !

initializePreRenderAction
	" Private - Initialize the receiver's pre-render action. "

	preRenderAction :=

	(DomainVelocity sphereAt: #(0 0 0) radius: 0.2 )
	, (DomainColor from: #(0 0 0) to: #(255 255 255) )
	, (ParticleSize x: 1.5)
	, ParticleCopyVertexB new

	, ((DomainSource sphereAt: #(0 0 0) radius: 6 ) rate: 2)
	, ((ParticleOrbitPoint x: 0 y: 0 z: 0) magnitude: 0.05)

	, (DomainSinkInside sphereAt: #(0 0 0) radius: 1 )
	, (DomainSinkOutside sphereAt: #(0 0 0) radius: 8 )
	, ParticleMove new! !



!Rain methods !

bitmapName
	" Private - Returns the receiver's bitmap name"

	^'buble'!
  
initializePreRenderAction
	" Private - Initialize the receiver's pre-render action. "

	preRenderAction :=

	(ParticleVelocity x: 0 y: 0 z: 0)
	, (ParticleSize x: 1.5)
	, ParticleCopyVertexB new

	, (DomainColor sphereAt: #(240 240 250) radius: 25 )
	, ((DomainSource
		rectangleAt: #(-10 -10 12) u: #(20 0 0) v: #(0 20 0) )
		rate: 10)
	, (ParticleGravity x: -0.002 y: 0.001 z: -0.1)

	, (DomainRandomAcceleration blobAt: #(0 0 -0.01) stdev: 0.003)
	, ((DomainBounce planeAt: #(0 0 -7) normal: #(0 0 1) )
		friction: 0.3; resilience: 0.3; cutoff: 0)
	, (ParticleKillOld value: 50 )
	, ParticleMove new! !



!Shape methods !

initializePreRenderAction
	" Private - Initialize the receiver's pre-render action. "

	| dim x y z xx yy zz dx dy dz |
	dim := (self maxParticles raisedTo: 0.3333333) asInteger.
	xx := 8. yy := 12. zz := 9.
	dx := xx * 2 / dim.	dy := yy * 2 / dim.	dz := zz * 2 / dim.
	preRenderAction := ParticleAction empty.

	z := zz negated.
	0 to: dim do: [:k|
		z := z + dz.	y := yy negated.
		0 to: dim do: [:i|
			y := y + dy.	x := xx negated.
			0 to: dim do: [:j|
				x := x + dx.
				preRenderAction := preRenderAction
					, (DomainVelocity blobAt: #(0 0 0) stdev: 0.001)
					, (ParticleColor
						red: z * 0.05 + 0.5 * 255 
						green: x * 0.05 + 0.5 * 255 
						blue: x * 0.05 + 0.5 * 255 
						alpha: 1)
					, (ParticleVertex x: x y: y z: z)
			].
		].
	].! !



!Waterfall methods !
  
initializePreRenderAction
	" Private - Initialize the receiver's pre-render action. "

	preRenderAction :=

	(DomainVelocity blobAt: #(0.03 -0.001 0.01) stdev: 0.002)
	, (DomainColor from: #(20 200 255) to: #(255 255 255) )
	, (ParticleSize x: 1.5)
	, ParticleCopyVertexB new

	, ((DomainSource
		lineFrom: #(-5 -1 8) to: #(-5 - 3 8) )
		rate: 2"particles per evaluation")
	, (ParticleGravity z: -0.0098)
	, (ParticleKillOld value: 300)

	, ((DomainBounce rectangleAt: #(-7 4 7) u: #(3 0 0) v: #(0 3 0) )
		friction: 0; resilience: 0.35; cutoff: 0)
	, ((DomainBounce sphereAt: #(-4 -2 4) radius: 0.2 )
		friction: 0; resilience: 0.5; cutoff: 0)
	, ((DomainBounce sphereAt: #(-3.5 0 2) radius: 2 )
		friction: 0; resilience: 0.5; cutoff: 0)
	, ((DomainBounce sphereAt: #(3.8 0 0) radius: 2 )
		friction: 0; resilience: 0.5; cutoff: 0)
	, ((DomainBounce planeAt: #(0 0 0) normal: #(0 0 1) )
		friction: -0.01; resilience: 0.5; cutoff: 0)

	, (DomainSinkOutside sphereAt: #(0 0 0) radius: 20 )
	, ParticleMove new! !



!ParticleExampleEditor methods !
   
changeExampleMenu
    " Private - Returns the examples alternatives menu."

	| menu |
	menu := Menu new
		owner: self;
		title: '&Example...';
		yourself.
	ParticleExample allSubclasses do: [:each|
		menu
			appendItem: each name
			action: (Message
				receiver: self
				selector: #changeExampleToOneOf:
				arguments: (Array with: each)).
	].
	^menu!
   
changeExampleToOneOf: aClass
    " Private - Change the example to one instance of aClass. "

	self example: aClass new.!

colorOf: aParticle
	" Private - Returns the color used to render aParticle. "

	| aParticleVector scale |
	scale := self example colorScale.
	aParticleVector := aParticle color. 
	^ RGBColor
		red: (aParticleVector x * scale) truncated
		green: (aParticleVector y * scale) truncated
		blue: (aParticleVector z * scale) truncated!
  
demo
    "Returns the instance var of the receiver
     for external manipulation"
    ^demo!

drawParticle: aParticle in: aWorld
	" Private - Draw aParticle in aWorld. "

	| aPolygon |
	aPolygon := self polygonIn: aWorld for: aParticle.
	aPolygon notNil ifTrue: [ aPolygon addOnce. ].!

drawParticles: particles
	" Private - Particles in the receiver's group has changed, and need to be rendered."

	| aWorld |
	(particles isEmpty
	or: [ (aWorld := demo world) isNil ])
	ifTrue: [ ^self ].
	
	particles particleDo: [:particle|
		self drawParticle: particle in: aWorld
	]!
  
example
    "Returns the instance var of the receiver
     for external manipulation"
    ^example!
  
example: aParticleExample
    "Sets the example of the receiver."

	self polygon: nil.
	example release.
    example := aParticleExample.
	example
		when: #needsToDrawParticles:
		send: #drawParticles:
		to: self.!
 
geBitmapOf: aParticle in: aWorld
	" Private - Returns a GenesisBitmap used to draw aParticle in aWorld. "

	^self example defaultBitmapIn: aWorld!
   
geVector: aParticleVector
	" Private - Transform a particle vector in a Genesis Vector."

	^GenesisVector3D
		x: aParticleVector x
		y: aParticleVector z	"WARNING: Must be converted to genesis coordinate system."
		z: aParticleVector y!
  
initMenu
	" Private - Do nothing..."

	^nil!
 
inspectExample
	" Private - Inspect the example. "

	self example inspect.!
  
intensityOf: aParticle
	" Private - Returns the intensity used to render aParticle. "

	^aParticle alpha * 255!
  
menu
    "Returns the menu of the receiver."

	| menu |
	menu := Menu new
		owner: self;
		title: '&Particles';
		appendSubMenu: self changeExampleMenu;
		"appendItem: '&visit models' selector: #cameraVisitModels;"
		appendSeparator;
		appendItem: '&inspect' selector: #inspectExample;
		yourself.
	^menu!
   
open: mode on: anExample
	" Private - Open the receiver on anExample."

	"We currently use GenesisDemo as an editor."
	demo := (Smalltalk at: #GenesisDemo) new.
	anExample defaultBitmap. "force default bitmap loading"
	demo
		model: self;
		when: #closeApplication send: #release to: example;
		perform: mode;
		openFile: self worldFileName;
		startTracking;
		yourself.
	self example: anExample.!
  
openFullScreenOn: anExample
	"Open the receiver on anExample."

	^self open: #openFullScreen on: anExample!
  
openOn: anExample
	"Open the receiver on anExample."

	^self open: #open on: anExample!
  
polygon: aPolygon
	" Private - Sets the polygon of the receiver. "

	polygon release.
	polygon := aPolygon.!

polygonIn: aWorld
	" Private - Returns a polygon to display a particle in aWorld. "

	polygon notNil ifTrue: [ "reuse old polygon."
		polygon world: aWorld.
		^polygon
	].

	^polygon := aWorld newPolygon
		vertexUV: 0;
		texturedPoint;
		notOccludeOthersNorSelf;
		yourself!
  
polygonIn: aWorld for: aParticle
	" Private - Returns a polygon used to draw aParticle in aWorld. "

	| aPolygon |
	aPolygon := self polygonIn: aWorld.
	self setVertex: aPolygon vertex with: aParticle.
	aPolygon
		bitmap: (self geBitmapOf: aParticle in: aWorld);
		scale: (self scaleOf: aParticle).
	^aPolygon!
 
positionOf: aParticle
	" Private - Returns the render position of aParticle. "

	self example drawScale isNil ifTrue: [
		^"self geVector:" aParticle position
	].
	^(self geVector: aParticle position) * self example drawScale!

preRender: seconds
	"PreRender the receiver on it's environment."

	| aWorld |
	(self example notNil and: [ (aWorld := demo world) notNil ])
	ifTrue: [
		(self polygonIn: aWorld) bitmap:
			(self example defaultBitmapIn: aWorld).
	].!
  
scaleOf: aParticle
	" Private - Returns the scale used to render aParticle. "

	^self example bitmapScale * aParticle extent x!
  
setVertex: aVertex with: aParticle
	" Private - Set aVertex from aParticle. "

	aVertex
		xyz: (self positionOf: aParticle);
		color: (self colorOf: aParticle);
		intensity: (self intensityOf: aParticle);
		yourself.
!
  
step: seconds
	"Step the receiver on it's environment."

	self example notNil ifTrue: [
		self example draw.
	].!
  
worldFileName
	" Private - Returns the world file name of the demo. "

	^'levels\Particles.bsp'! !



!ParticleHandle methods !
  
asParameter
	"Answer the receiver in a form suitable for passing as a parameter to a host procedure call."

	^self handle asParameter!
   
handle
    "Returns the instance var of the receiver
     for external manipulation"

	handle isNil ifTrue: [ self initializeHandle ].
    ^handle!

handle: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    handle := anObject!
   
initializeHandle
	" Private - Initializes the receiver's handle"

	handle := self implementedBySubclass.!

isValid
	"Returns true when the receiver's pointer is not nil."

	^handle notNil "valid pointer."!
   
release
	"Free the resource associated with the receiver."

	handle notNil ifTrue: [ self releaseHandle ].
	^super release!
 
releaseHandle
	" Private - Free the resources associated with the receiver's handle.
	Warning: The default action is to CLEAR the handle.
	This method can be refined by the subclasses. "

	handle := nil.! !



!ParticleGroup class methods !
   
constructEventsTriggered
	"Private - answer the set of events that instances of the receiver can trigger."

    ^super constructEventsTriggered
		add: #needsContents;
		add: #aboutToRender;
		add: #particlesChanged:;
		yourself!
 
defaultMaxParticle
	" Private - Returns the default maximum particle count. "

	^4000!
   
maxParticles: maxParticles
    "Returns an instance of the receiver."

	^self new
		initialize: maxParticles;
		yourself!
  
new: maxParticles
    "Returns an instance of the receiver."

	^self maxParticles: maxParticles! !



!ParticleGroup methods !
   
draw
	" Private - Draw the receiver's particles using darwCallback. "

	self dll pDrawCallbackGroup: self drawCallBack asParameter.!
 
drawAll: count particles: particlePtr
	" Private - Must draw the particles at particlePtr. "

	self
		triggerEvent: #particlesChanged:
		with: (ParticlesArray
			new: count
			atAddress: particlePtr asExternalAddress)!
   
drawCallBack
    "Returns the instance var of the receiver
     for external manipulation"

	drawCallBack isNil ifTrue: [ self initializeDrawCallBack ].
    ^drawCallBack!

drawCallBack: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    drawCallBack := anObject!
   
initialize: maxParticles
    " Private - Initializes the receiver to support maxParticles. "

	self handle: (self dll
		pGenParticleGroups: 1 asParameter
		maxParticles: maxParticles asParameter).

	self triggerEvent: #needsContents.!
   
initializeDrawCallBack
	" Private - Initializes the receiver's drawCallBack.
	Initialize a callback sending #drawParticle: to the receiver. "

	self drawCallBack: (
		self dll class
			newCallBack: #DrawCallBack
			sending: #drawAll:particles:
			to: self)!
   
initializeHandle
	" Private - Initializes the receiver's handle"

	self initialize: self class defaultMaxParticle.!
  
initializePreDrawActions
	" Private - Initializes the receiver's preDrawActions"

	preDrawActions := OrderedCollection new.!
 
makeCurrent
	"Make the receiver the current group."

	self dll pCurrentGroup: self asParameter.!
 
maxParticles: maxParticles
    " Sets the receiver to support maxParticles. "

	self dll pSetMaxParticles: maxParticles asParameter.!

preDrawActions
    "Returns the instance var of the receiver
     for external manipulation"

	preDrawActions isNil ifTrue: [ self initializePreDrawActions ].
    ^preDrawActions!

preDrawActions: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    preDrawActions := anObject!
   
prepareRender
	" Private - Prepare the receiver to be rendered. "

	self triggerEvent: #aboutToRender.
	self preDrawActions do: [:each| each value: self ].!

release
	" Release the receiver. "

	drawCallBack release.
	^super release!
 
releaseHandle
	" Private - Free the resources associated with the receiver's handle. "

	self isValid ifTrue: [
		self dll
			pDeleteParticleGroups: self asParameter
			count: 1 asParameter
	].
	super releaseHandle.!

render
	" Render the receiver. "

	self
		makeCurrent;
		prepareRender;
		draw! !



!ParticleStructure class methods !
   
atAddress: anExternalAddress
	"Returns an instance of the receiver at anExternalAddress."
	
	^self new
		initializeStructure: anExternalAddress;
		yourself!
   
from: aStructure
	"Returns an instance of the receiver set with aStructure."
	
	^self new
		structure: aStructure;
		yourself!
 
with: aStructure
	"Returns an instance of the receiver with aStructure."
	
	^self from: aStructure! !



!ParticleStructure methods !

initializeStructure
	" Private - Initializes the receiver's structure"

	self structure: (self dll structureNamed: self structureName)!
  
initializeStructure: particlePtr
	" Private - Initializes the receiver's structure"

	self structure: (self dll class
		structureNamed: self structureName
		atAddress: particlePtr asExternalAddress)!

structure
    "Returns the instance var of the receiver
     for external manipulation"

	structure isNil ifTrue: [ self initializeStructure ].
    ^structure!

structure: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    structure := anObject!
 
structureName
	" Private - Must return the structure name of the receiver."

	^self implementedBySubclass!
   
vectorFrom: aStructure
	" Private - Returns a vector from aStructure. "

	^ParticleVector with: aStructure! !



!Particle methods !
 
age
    "Returns the age of the receiver."

    ^self stFloat: self structure age!
   
alpha
    "Returns the alpha of the receiver."

    ^self stFloat: self structure alpha!
 
color
	" Returns the receiver's color (as a vector). "

	^self vectorFrom: self structure color!
 
extent
	" Returns the receiver's extent (the size of the particle). "

	^self vectorFrom: self structure extent!
 
position
	" Returns the receiver's position. "

	^self vectorFrom: self structure position!
  
positionB
	" Returns the receiver's positionB. "

	^self vectorFrom: self structure positionB!
   
structureName
	" Private - Must return the structure name of the receiver."

	^'Particle'!
   
velocity
	" Returns the receiver's velocity. "

	^self vectorFrom: self structure velocity!
  
velocityB
	" Returns the receiver's velocityB. "

	^self vectorFrom: self structure velocityB! !



!ParticlesArray class methods !
  
new: size
	"Returns an instance of the receiver of size size"

	^self new
		count: size;
		yourself!
   
new: count atAddress: anExternalAddress
	"Returns an instance of the receiver at anExternalAddress of size count."

	^(self atAddress: anExternalAddress)
		count: count;
		yourself! !



!ParticlesArray methods !
   
at: index
	"Answer the particle at index."

	(index between: 1 and: self size) ifFalse: [
		^self errorInBounds: index
	].
	^Particle from: (self structure at: index)!
   
count
	"Answer the number of elements contained in the receiver."

	count isNil ifTrue: [ count := 0 ].
	^count!

count: anInteger
	" Private - Set the particle count of the receiver. "

	count := anInteger!

do: aBlock
        "Answer the receiver.  For each element in the receiver,
         evaluate aBlock with that element as the argument."
    | index size |
    index := 1.
    size := self size.
    [index > size]
        whileFalse: [
            aBlock value: (self at: index).
            index := index + 1]!
   
errorInBounds: anInteger
        "Private - Report an error to the effect that
         anInteger is an invalid index for the receiver."
    ^self
        error: 'index: ',
            anInteger printString,
            ' is outside of collection bounds'!
   
isEmpty
	"Answer true if the receiver id empty."

	^self count = 0!
  
particleDo: aBlock
	"For each element in the receiver,
		evaluate aBlock with aParticle as the argument.
	WARNING: a speed optimization is done
	 and the particle argument will be the SAME object
	 at each evaluation of aBlock.
	Then the argument is NOT valid between evaluations.
	Answer the receiver."

    | index current |
	self count = 0 ifTrue: [ ^self ].
    current := self at: (index := 1).
    [index > count] whileFalse: [
		current structure: (structure at: index).
		aBlock value: current.
		index := index + 1.			"*Debug* index > 50 ifTrue: [ ^self]."
	].

	"*Debug*
	(self at: 1) velocity displayAt: 10@10.
	"!
   
size
	"Answer the number of elements contained in the receiver."

	^self count!
  
structure: aParticles
    "Sets the instance var of the receiver with anObject. (No checking is done)"

	aParticles notNil "we will use the contained open array of particles."
	ifTrue: [ super structure: aParticles array ]
	ifFalse: [ super structure: aParticles ].!
 
structureName
	" Private - Must return the structure name of the receiver."

	^'Particles'! !



!ParticleVector methods !
   
printOn: aStream
	"Append the ASCII representation of the receiver to aStream."

	aStream
		nextPut: $[;
		yourself.
	#(x y z) do: [:selector|
		aStream space.
		[	aStream nextPutAll: (self perform: selector) printString
		] on: Error do: [ aStream nextPutAll: 'invalid' ].
	].
	aStream space;nextPut: $]!

structureName
	" Private - Must return the structure name of the receiver."

	^'pVector'!

x
    "Returns the x of the receiver."

    ^self stFloat: self structure x!
 
y
    "Returns the y of the receiver."

    ^self stFloat: self structure y!
 
z
    "Returns the z of the receiver."

    ^self stFloat: self structure z! !



!ParticleDLL class methods !
   
callbackTable
	" Private - Returns the callback shape table for defining callback handlers."

	^#(
	"(callBackName (arguments...) returnType callingConvertion)"

	(DrawCallBack ( long ulong ) none api)
	)!

constants
	"Returns the constants dictionary."

	| constants |
	constants := self constantsDictionary.
	constants size = 0 ifTrue: [ self initializeConstants ].
	^constants!
 
constantsDictionary
	" Private - Returns the constants dictionary. "

	^Smalltalk
		at: #ParticleConstants
		ifAbsentPut: [ Dictionary new ]!
  
fileName
		"Answer the receiver's file name."

	^'Particle.dll'!
 
initializeConstants
	" Private - Initialize the constants dictionary.
	ParticleDLL initializeConstants.
	"

	self
		initializeDomainConstants;
		yourself.!
  
initializeDomainConstants
	" Private - Initialize the particle domain constants. "

	self constantsDictionary
		at: 'PDPoint' put: 0; "Single point"
		at: 'PDLine' put: 1; "Line segment"
		at: 'PDTriangle' put: 2; "Triangle"
		at: 'PDPlane' put: 3; "Arbitrarily-oriented plane"
		at: 'PDBox' put: 4; "Axis-aligned box"
		at: 'PDSphere' put: 5;
		at: 'PDCylinder' put: 6;
		at: 'PDCone' put: 7;
		at: 'PDBlob' put: 8; "Gaussian blob"
		at: 'PDDisc' put: 9; "Arbitrarily-oriented disc"
		at: 'PDRectangle' put: 10; "Rhombus-shaped planar region"
		yourself.!
 
initializeGeneralStructures
	" Private - Initialize the general.h Structures. "

	self
		registerType: 'Particle'
		as:#(
			(position		'pVector')
			(positionB	'pVector')
			(extent			'pVector') "Size of the particle"
			(velocity		'pVector')
			(velocityB	'pVector') "Used to compute binormal, normal, etc."
			(color 	'pVector') "Color must be next to alpha so glColor4fv works."
			(alpha 	float)		 "This is both cunning and scary."
			(age		float)
		);
		registerType: 'Particles'
		as:#( "Open array of particles..."
			(array (0 'Particle'))
		);
		yourself.!
   
initializeStructures
	"Private - Initializes the structures for the interfase.
	ParticleDLL initializeStructures
	"

	super initializeStructures.

	self
		initializeVectorStructures;
		initializeGeneralStructures;
		yourself.!

initializeVectorStructures
	" Private - Initialize the pVector.h Structures. "

	self
		registerType: 'pVector'
		as:#( (x float) (y float) (z float) );
		yourself.!
 
newCallBack: callBackName sending: selector to: aReceiver
	"Returns a CallBack object for sending message with selector to aReceiver.
	callBackName must be a valid name defined in callbackTable."

	| tuple aMessage parameterTypes |
	tuple := self callbackTable
		detect: [:one| one first = callBackName ]
		ifNone: [ ^self error: 'Invalid callback name.'].

	"check selector arguments with parameterTypes size"
	parameterTypes := tuple at: 2.
	(selector asString select: [:c| c = $: ]) size = parameterTypes size
	ifFalse: [ ^self error: 'Invalid selector (Must have ',parameterTypes size asString,' arguments.' ].

	aMessage := Message new
		receiver: aReceiver;
		selector: selector;
		yourself.
	^CallBack
		registerMessage: aMessage
		parameterTypes: parameterTypes
		returnType: (tuple at: 3)
		callingConvention: (tuple size > 3 ifTrue: [ tuple at: 4 ] ifFalse: [#api]).!
 
registerType: aStrucName as: aDefinitionArray
	"Private - Register a new SelfDefinedStructure with
	name aStrucName with type aDefinitionArray.
	"

	| array |
	array := aDefinitionArray collect: [:tuple|
		(tuple at: 2) = #float
		ifTrue: [ Array with: tuple first with: 4 ]
		ifFalse: [ tuple]
	].
	^super register: array as: aStrucName! !



!ParticleDLL methods !
   
cppFloat: aNumber
	" Private - Transform aNumber into a cpp float parameter. "

	| result theFloat |
	Float clearExceptionFlags. "clear exception flags because sometimes are signalled incorrectly."
	theFloat := aNumber asFloat.
	result := OLEStructure externalFloatFromInternalFloat: theFloat.
	^result!
  
cppFloats: anArray
	" Private - Transform anArray (of numbers) into a cpp float array to be used as valid API parameters. "

	| result |
	result := Array new: (9 max: anArray size).
	result atAllPut: self zero asParameter.
	1 to: anArray size do: [:index|
		result
			at: index
			put: (self cppFloat: (anArray at: index)) asParameter
	].
	^result!
 
defaults
	" Private - Returns an array with default float values. "

	^#( ) "we use an empty array to fill all arguments with zero as default."!
 
epsilon
	"Returns the smaller number supported."

	^1.0e-3 "1.0e-3 = (1/1000)"
	"
	Some messages have parameters with a default value of pEPS.
	pEPS is a very small floating point constant that is most often used
	 as the default value of the epsilon parameter to actions whose
	 influence on a particle is relative to the inverse square of its
	 distance from something.
	If that distance is very small, the amount of influence approaches infinity.
	Since all actions are computed using Euler's method, this can
	 cause unsatisfying results in which particles are accelerated
	 way too much.
	So this epsilon parameter is added to the distance before taking
	 its inverse square, thus keeping the acceleration within
	 reasonable limits.
	By varying epsilon, you specify what is reasonable.
	Larger epsilon make particles accelerate less.
	"!
  
maxFloat
	"Returns the greater number supported."

	^1.0e16!
 
maxInt
	"Returns the greater integer supported."

	^16r7FFFFFFF!
 
pAvoid: magnitude epsilon: epsilon lookAhead: lookAhead dtype: dtype
	"Steer particles away from a domain of space."

	^self
		pAvoid: magnitude epsilon: epsilon
		lookAhead: lookAhead dtype: dtype
		with: self defaults!
  
pAvoid: magnitude
	epsilon: epsilon
	lookAhead: lookAhead
	dtype: dtype
	 with: arguments
	"Steer particles away from a domain of space."

	| parameters |
	parameters := ReadStream on: (self cppFloats: arguments).
	^self
		pAvoid: magnitude epsilon: epsilon
		lookAhead: lookAhead dtype: dtype
		a0: parameters next a1: parameters next a2: parameters next
		a3: parameters next a4: parameters next a5: parameters next
		a6: parameters next a7: parameters next a8: parameters next!
  
pBounce: friction resilience: resilience cutoff: cutoff dtype: dtype
	"Bounce particles off a domain of space."

	^self
		pBounce: friction resilience: resilience
		cutoff: cutoff dtype: dtype
		with: self defaults!
   
pBounce: friction resilience: resilience
	cutoff: cutoff dtype: dtype
	a0: a0 a1: a1 a2: a2
	a3: a3 a4: a4 a5: a5
	a6: a6 a7: a7 a8: a8
	"Bounce particles off a domain of space.
	Particles are tested to see whether they will pass from being outside
	 the specified domain to being inside it if the next #pMove:... action
	 were to occur now. If they would pass through the surface of the
	 domain, they are instead bounced off it. That is, their velocity vector
	 is decomposed into components normal to the surface and tangent
	 to the surface. The direction of the normal component is reversed,
	 and the components are recomposed into a new velocity heading
	 away from the surface.
	The normal component is multiplied by the resilience parameter
	 and the tangential component, if its magnitude is greater than
	 cutoff, is multiplied by (1 - the friction parameter) when being
	 composed into the new velocity vector.
	The cutoff parameter can allow particles to glide smoothly along
	 a surface without sticking.
	NOTES: Since particles are tested to see whether they would pass
	 through the domain if pMove were called now, it is best to have
	 pBounce be the last action that modifies a particle's velocity
	 before calling pMove. Also, actions such as pRandomDisplace
	 and pVortex that modify a particle's position directly, rather
	 than modifying its velocity vector, may yield unsatisfying results
	 when used with pBounce.
	At present the only domains for which pBounce is implemented
	 are PDSphere, PDRectangle, DTriangle, PDDisc and PDPlane.
	For spheres, the particle is always forced out of the sphere.
	For planes, triangles and discs, the particles bounce off either
	 side of the surface. For rectangles, particles bounce off either
	 side of the diamond-shaped patch whose corners are o, o+u,
	 o+u+v, and o+v.
	See the documentation of Domains for an explanation of the
	 other arguments.
	#pBounce:... doesn't work correctly with small time step sizes
	 for particle sliding along a surface. The friction and resilience
	 parameters should not be scaled by dt, since a bounce happens
	 instantaneously. On the other hand, they should be scaled
	 by dt because particles sliding along a surface will hit more
	 often if dt is smaller. If you have any suggestions, let me know."
	<api: '?pBounce@@YAXMMMW4PDomainEnum@@MMMMMMMMM@Z'
		ulong ulong ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		none>
	^self invalidArgument!

pBounce: friction resilience: resilience cutoff: cutoff dtype: dtype with: arguments
	"Bounce particles off a domain of space."

	| parameters |
	parameters := ReadStream on: (self cppFloats: arguments).
	^self
		pBounce: friction resilience: resilience
		cutoff: cutoff dtype: dtype
		a0: parameters next a1: parameters next a2: parameters next
		a3: parameters next a4: parameters next a5: parameters next
		a6: parameters next a7: parameters next a8: parameters next!

pColor: red green: green blue: blue
	"Specify the color of new particles.
	All new particles will have the color red, green, blue, 0."

	^self pColor: red green: green blue: blue alpha: self zero asParameter
	!
 
pColor: red green: green blue: blue alpha: alpha
	"Specify the color of new particles.
	All new particles will have the color red, green, blue, alpha."

	<api: '?pColor@@YAXMMMM@Z' ulong ulong ulong ulong none>
	^self invalidArgument!
 
pColorD: alpha dtype: aPDomainEnum
	"Specify the color domain of new particles.
	All new particles will have a color chosen randomly from within the specified domain
	 and will have an alpha value (usually used for transparency) of alpha."

	^self pColorD: alpha dtype: aPDomainEnum with: self defaults!

pColorD: alpha dtype: aPDomainEnum
	a0: a0 a1: a1 a2: a2
	a3: a3 a4: a4 a5: a5
	a6: a6 a7: a7 a8: a8
	"Specify the color domain of new particles.
	All new particles will have a color chosen randomly from within the specified domain
	 and will have an alpha value (usually used for transparency) of alpha."

	<api: '?pColorD@@YAXMW4PDomainEnum@@MMMMMMMMM@Z' ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		none>
	^self invalidArgument!
   
pColorD: alpha dtype: aPDomainEnum with: arguments
	"Specify the color domain of new particles.
	All new particles will have a color chosen randomly from within the specified domain
	 and will have an alpha value (usually used for transparency) of alpha."

	| parameters |
	parameters := ReadStream on: (self cppFloats: arguments).
	^self pColorD: alpha dtype: aPDomainEnum
		a0: parameters next a1: parameters next a2: parameters next
		a3: parameters next a4: parameters next a5: parameters next
		a6: parameters next a7: parameters next a8: parameters next!
  
pCopyGroup: groupNum
	"Copy particles from the specified group into the current group."

	^self pCopyGroup: groupNum
		index: 1 asParameter
		count: self pMaxInt asParameter!
 
pCopyVertexB
	"Set the secondary position from current position."

	^self pCopyVertexB: true asParameter!

pCopyVertexB: copyPos
	"Set the secondary position from current position.
	If copyPos is true, sets the positionB of each particle
	 in the current particle group to be equal to the current
	 position of that particle. This makes each particle 'remember'
	 this position so it can later return to it using #pRestore:..."

	^self pCopyVertexB: copyPos copyVel: false asParameter!
   
pCopyVertexB: copyPos copyVel: copyVel
	"Set the secondary position from current position.
	If copyPos is true, sets the positionB of each particle
	 in the current particle group to be equal to the current
	 position of that particle. This makes each particle 'remember'
	 this position so it can later return to it using #pRestore:...
	If copyVel is true, sets the velocityB of each particle
	 in the current particle group to be equal to the current
	 velocity of that particle. This is useful for computing
	 the orientation of the particle when rendering it using
	 #pDrawGroupl:..."

	<api: '?pCopyVertexB@@YAX_N0@Z' boolean boolean none>
	^self invalidArgument!
   
pCurrentGroup: groupNum
	"Change which group is current.
	Makes groupNum be the current particle group to which all
	 actions and commands apply."

	<api: '?pCurrentGroup@@YAXH@Z' long none>
	^self invalidArgument!

pDamping: x y: y z: z
	"Simulate air by slowing down particle velocities."

	^self pDamping: x y: y z: z
		vlow: self zero asParameter
		vhigh: self pMaxFloat asParameter!

pDamping: x y: y z: z vlow: vLow vhigh: vHigh
	"Simulate air by slowing down particle velocities.
	If a particle's velocity magnitude is within vlow and vhigh, then multiply each component of the velocity by the respective damping constant. Typically, the three components of damping will have the same value.
	NOTES: There are no bounds on the damping constants. Thus, by giving values greater than 1.0 they may be used to speed up particles in stead of slow them down."

	<api: '?pDamping@@YAXMMMMM@Z'
		ulong ulong ulong ulong ulong
		none>
	^self invalidArgument!
 
pDeleteActionList: actionListNum
	"Delete one action list."

	^self
		pDeleteActionLists: actionListNum
		count: 1 asParameter!

pDeleteParticleGroups: groupNum
	"Delete one particle group."

	^self
		pDeleteParticleGroups: groupNum
		count: 1 asParameter!

pDeleteParticleGroups: groupNum count: count
	"Delete one or more consecutive particle groups.
	Deletes count particle groups, with groupNum being the particle group
	 number of the first one. The groups must be numbered sequentially,
	 and must all exist. This removes the specified particle groups
	 from existence. It does not merely change the number of existing
	 particles or the maximum size of the group."

	<api: '?pDeleteParticleGroups@@YAXHH@Z'
		long long
		none>
	^self invalidArgument!
  
pDrawCallbackGroup: pCallback
	"Draw the current group using callback function(called once for all the particles of the group)."

	<api: '?pDrawCallbackGroup@@YAXP6GXHPAUParticle@@@Z@Z'
		ulong
		none>
	^self invalidArgument!
 
pDrawCallbackParticles: pCallback
	"Draw all the particles in the current group using callback function(for each particle)."

	<api: '?pDrawCallbackParticles@@YAXP6GXHPAUParticle@@@Z@Z'
		ulong
		none>
	^self invalidArgument!
 
pEPS
	"Returns the smaller number supported as a parameter."

	^self cppFloat: self epsilon!
 
pExplosion: centerX centerY: centerY centerZ: centerZ
	velocity: velocity
	magnitude: magnitude
	stdev: stdev
	"An Explosion.
	Causes an explosion by accelerating all particles away from the center.	"

	^self
		pExplosion: centerX centerY: centerY centerZ: centerZ
		velocity: velocity magnitude: magnitude
		stdev: stdev epsilon: self pEPS asParameter
		age: self pMaxFloat asParameter!
  
pExplosion: centerX centerY: centerY centerZ: centerZ
	velocity: velocity
	magnitude: magnitude
	stdev: stdev
	epsilon: epsilon
	age: age
	"An Explosion.
	Causes an explosion by accelerating all particles away from the center.
	Particles are accelerated away from the center by an amount
	 proportional to magnitude. As with most acceleration actions,
	 the amount of acceleration falls off inversely with r2.
	 But when r is small, the acceleration would be infinite, so epsilon
	 is always added to r.
	The shock wave of the explosion has a gaussian magnitude.
	The center of the gaussian travels outward from the center at the
	 specified velocity. So at a given time step, particles at a distance
	 (velocity * age) from center will receive the most acceleration,
	 and particles not at the peak of the shock wave will receive
	 a lesser acceleration.
	The shock wave has a standard deviation of stdev, which is the
	 sharpness or broadness of the strength of the wave.
	age is used to calculate the radius of the shock wave.
	For pExplosion calls in action lists, age is the initial age of the
	 explosion. It is incremented by dt after each call.
	For immediate mode, age is the current age of the explosion,
	 and it is up to the application to increment the age parameter
	 for each send of #pExlosion:...
	NOTE: Since the current radius of the shock wave is given by
	 (velocity * age) you can set up a standing wave by setting
	 velocity to the desired radius of the wave and age to 1.0.
	This only works in immediate mode.
	"
	<api: '?pExplosion@@YAXMMMMMMMM@Z'
		ulong ulong ulong
		ulong ulong ulong
		ulong ulong
		none>
	^self invalidArgument!
 
pFollow
	"Accelerate toward the next particle in the group.
	This allows snaky effects where the particles follow each other.
	"
	^self pFollow: 1 asParameter!
 
pFollow: magnitude
	"Accelerate toward the next particle in the group.
	This allows snaky effects where the particles follow each other.
	"
	^self pFollow: magnitude
		epsilon: self pEPS asParameter
		maxRadius: self pMaxFloat asParameter!
   
pFollow: magnitude epsilon: epsilon
	"Accelerate toward the next particle in the group.
	This allows snaky effects where the particles follow each other.
	"
	^self pFollow: magnitude
		epsilon: epsilon
		maxRadius: self pMaxFloat asParameter!

pFollow: magnitude epsilon: epsilon maxRadius: maxRadius
	"Accelerate toward the next particle in the group.
	This allows snaky effects where the particles follow each other.
	Each particle is accelerated toward the next particle in the group
	 by an amount proportional to magnitude.
	As with most acceleration actions, the amount of acceleration falls
	 off inversely with r2. But when r is small, the acceleration would
	 be infinite, so epsilon is always added to r.
	maxRadius defines the sphere of influence of this action.
	No particle further than maxRadius from its predecessor is affected.
	NOTES: The #pFollow:.. action does not affect the last particle
	 in the group. This allows controlled effects where the last particle
	 in the group is killed after each time step and replaced by
	 a new particle at a slightly different position.
	See #pKillOld:.. to learn how to kill the last particle in the group
	 after each step.
	"
	<api: '?pFollow@@YAXMMM@Z'
		ulong ulong ulong
		none>
	^self invalidArgument!

pGenActionList
	"Generate an empty action list.
	Returns the action list number."

	^self pGenActionLists: 1 asParameter!
   
pGenParticleGroup
	"Create a particle group, with no articles allocated."

	^self pGenParticleGroups: 1 asParameter!
 
pGenParticleGroups: count
	"Create particle groups, each with no articles allocated."

	^self pGenParticleGroups: count maxParticles: 0 asParameter!
 
pGenParticleGroups: count maxParticles: maxParticles
	"Create particle groups, each with maxParticles allocated.
	Generates count new particle groups and returns the particle
	 group number of the first one. The groups are numbered
	 sequentially, beginning with the number returned.
	Each particle group is set to have at most maxParticles particles.
	Use #pSetMaxParticles:... to change this.
	Particle group numbers of groups that have been deleted
	 (using #pDeleteParticleGroups:...) may be reused by #pGenParticleGroups:...
	"
	<api: '?pGenParticleGroups@@YAHHH@Z' long long ulongReturn>
	^self invalidArgument!

pGetParticles: index0 count: count position: positionHolder
	"Copy the position of particles from the current group to application memory.
	WARNING: As with all arrays in C, the index of the first particle
	 is zero."

	^self pGetParticles: index0
		count: count
		position: positionHolder
		color: nil asParameter
		velocity: nil asParameter
		size: nil asParameter
		age: nil asParameter!
   
pGravitate
	"Accelerate each particle toward each other particle.
	Each particle is accelerated toward each other particle in
	 the group by an amount proportional to 1.
	"
	^self pGravitate: 1 asParameter!
 
pGravitate: magnitude
	"Accelerate each particle toward each other particle.
	Each particle is accelerated toward each other particle in
	 the group by an amount proportional to magnitude.
	"
	^self pGravitate: magnitude
		epsilon: self pEPS asParameter
		maxRadius: self pMaxFloat asParameter!
   
pGravitate: magnitude epsilon: epsilon
	"Accelerate each particle toward each other particle.
	Each particle is accelerated toward each other particle in
	 the group by an amount proportional to magnitude.
	"
	^self pGravitate: magnitude
		epsilon: epsilon
		maxRadius: self pMaxFloat asParameter!

pGravity
	"Accelerate particles in the (0, 0, -9.8) direction."

	^self
		pGravity: self zero asParameter
		y: self zero asParameter
		z: (self cppFloat: -98/10) asParameter!

pGravity: x y: y z: z
	"Accelerate particles in the given direction.
	The gravity acceleration vector is simply added to the velocity
	 vector of each particle at each time step.
	The magnitude of the gravity vector is the acceleration due
	 to gravity.
	For example, pGravity(0, 0, -9.8) specifies gravity in the
	 negative Z direction.
	"
	<api: '?pGravity@@YAXMMM@Z'
		ulong ulong ulong
		none>
	^self invalidArgument!
   
pJet: centerX centerY: centerY centerZ: centerZ
	"Accelerate particles that are near the center of the jet.
	For each particle, chooses an acceleration vector from the domain
	 and applies it to the particle's velocity.
	"
	^self
		pJet: centerX centerY: centerY centerZ: centerZ
		magnitude: 1 asParameter!
  
pJet: centerX centerY: centerY centerZ: centerZ magnitude: magnitude
	"Accelerate particles that are near the center of the jet.
	For each particle, chooses an acceleration vector from the domain
	 and applies it to the particle's velocity.
	"
	^self
		pJet: centerX centerY: centerY centerZ: centerZ
		magnitude: magnitude
		epsilon: self pEPS asParameter
		maxRadius: self pMaxFloat asParameter!
  
pJet: centerX centerY: centerY centerZ: centerZ
	magnitude: magnitude
	epsilon: epsilon
	maxRadius: maxRadius
	"Accelerate particles that are near the center of the jet.
	For each particle, chooses an acceleration vector from the domain
	 and applies it to the particle's velocity.
	The amount of acceleration applied is directly proportional to magnitude.
	As with most acceleration actions, the amount of acceleration falls
	 off inversely with r2. But when r is small, the acceleration would
	 be infinite, so epsilon is always added to r.
	The domain from which acceleration vectors are chosen is the
	 current velocity domain.
	maxRadius defines the sphere of influence of this action.
	No particle further than maxRadius from the center is affected.
	"
	<api: '?pJet@@YAXMMMMMM@Z'
		ulong ulong ulong
		ulong ulong ulong
		none>
	^self invalidArgument!
  
pKillOld: ageLimit
	"Remove old particles.
	Removes all particles older than ageLimit. "

	^self pKillOld: ageLimit killLessThan: false asParameter!

pKillOld: ageLimit killLessThan: killLessThan
	"Remove old particles.
	Removes all particles older than ageLimit.
	But if killLessThan is true, it instead removes all particles
	 newer than ageLimit.
	ageLimit is not clamped, so negative values are ok.
	This can be used in conjunction with #pStartingAge: -n
	 to create and then kill a particular set of particles.
	NOTES: In order to kill a particular particle, set #pStartingAge:
	 to a number that will never be a typical age for any
	 other particle in the group, for example -10
	Then emit the particle using pSource or pVertex.
	Then do the rest of the particle actions and finally send
	 #pKillOld: -8.0 killLessThan: true
	 to kill the special particle because it is the only one
	 with an age less than -8.0.
	"
	<api: '?pKillOld@@YAXM_N@Z'
		ulong boolean
		none>
	^self invalidArgument!

pMatchVelocity
	"Modify each particles velocity to be similar to that of its neighbors.
	Each particle is accelerated toward the weighted mean of the velocities
	 of other particles in the group by an amount proportional to 1.
	"

	^self pMatchVelocity: 1 asParameter!
 
pMatchVelocity: magnitude
	"Modify each particles velocity to be similar to that of its neighbors.
	Each particle is accelerated toward the weighted mean of the velocities
	 of other particles in the group by an amount proportional to magnitude.
	"

	^self
		pMatchVelocity: magnitude
		epsilon: self pEPS asParameter
		maxRadius: self pMaxFloat asParameter!

pMaxFloat
	"Returns the greater number supported."

	^self cppFloat: self maxFloat!
  
pMaxInt
	"Returns the greater integer supported."

	^self maxInt!

pMove
	"Move particle positions based on velocities.
	This action actually updates the particle positions by adding
	 the current velocity to the current position.
	This is typically the last particle action performed in an
	 iteration of a particle simulation, and typically only occurs
	 once per action list.
	The velocity is multiplied by the time step length, dt,
	 before being added to the position.
	This implements Euler's method of numerical integration
	 with a constant, but specifiable step size.
	See #pTimeStep:... for more on varying the step size.
	"
	<api: '?pMove@@YAXXZ' none>
	^self invalidArgument!
  
pOrbitLine: x y: y z: z
	axisX: axisX axisY: axisY axisZ: axisZ
	"Accelerate toward the closest point on the given line.
	x@y@z and axis define an infinite line, where p can
	 be any point on the line and axis is any vector
	 parallel to the line."

	^self
		pOrbitLine: x y: y z: z
		axisX: axisX axisY: axisY axisZ: axisZ
		magnitude: 1 asParameter
		epsilon: self pEPS asParameter
		maxRadius: self pMaxFloat asParameter!

pOrbitLine: x y: y z: z
	axisX: axisX axisY: axisY axisZ: axisZ
	magnitude: magnitude
	"Accelerate toward the closest point on the given line.
	x@y@z and axis define an infinite line, where p can
	 be any point on the line and axis is any vector
	 parallel to the line."

	^self
		pOrbitLine: x y: y z: z
		axisX: axisX axisY: axisY axisZ: axisZ
		magnitude: magnitude
		epsilon: self pEPS asParameter
		maxRadius: self pMaxFloat asParameter!
 
pOrbitLine: x y: y z: z
	axisX: axisX axisY: axisY axisZ: axisZ
	magnitude: magnitude
	epsilon: epsilon
	"Accelerate toward the closest point on the given line.
	x@y@z and axis define an infinite line, where p can
	 be any point on the line and axis is any vector
	 parallel to the line."

	^self
		pOrbitLine: x y: y z: z
		axisX: axisX axisY: axisY axisZ: axisZ
		magnitude: magnitude
		epsilon: epsilon
		maxRadius: self pMaxFloat asParameter!

pOrbitLine: x y: y z: z
	axisX: axisX axisY: axisY axisZ: axisZ
	magnitude: magnitude
	epsilon: epsilon
	maxRadius: maxRadius
	"Accelerate toward the closest point on the given line.
	x@y@z and axis define an infinite line, where p can
	 be any point on the line and axis is any vector
	 parallel to the line.
	For each particle, this action computes the vector to
	 the closest point on the line, and accelerates the particle
	 in the vector direction.
	The amount of acceleration applied is directly proportional
	 to magnitude. As with most acceleration actions, the amount
	 of acceleration falls off inversely with r2.
	But when r is small, the acceleration would be infinite,
	 so epsilon is always added to r.
	maxRadius defines the infinite cylinder of influence of this action.
	No particle further than maxRadius from the line is affected. "

	<api: '?pOrbitLine@@YAXMMMMMMMMM@Z'
		ulong ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		none>
	^self invalidArgument!
  
pOrbitPoint: x y: y z: z
	"Accelerate toward the given center point.
	For each particle, this action computes the vector to the center point,
	 and accelerates the particle in the vector direction."

	^self
		pOrbitPoint: x y: y z: z
		magnitude: 1 asParameter
		epsilon: self pEPS asParameter
		maxRadius: self pMaxFloat asParameter!
 
pOrbitPoint: x y: y z: z
	magnitude: magnitude
	"Accelerate toward the given center point.
	For each particle, this action computes the vector to the center point,
	 and accelerates the particle in the vector direction."

	^self
		pOrbitPoint: x y: y z: z
		magnitude: magnitude
		epsilon: self pEPS asParameter
		maxRadius: self pMaxFloat asParameter!
  
pOrbitPoint: x y: y z: z
	magnitude: magnitude
	epsilon: epsilon
	"Accelerate toward the given center point.
	For each particle, this action computes the vector to the center point,
	 and accelerates the particle in the vector direction."

	^self
		pOrbitPoint: x y: y z: z
		magnitude: magnitude
		epsilon: epsilon
		maxRadius: self pMaxFloat asParameter!
 
pOrbitPoint: x y: y z: z
	magnitude: magnitude
	epsilon: epsilon
	maxRadius: maxRadius
	"Accelerate toward the given center point.
	For each particle, this action computes the vector to the center point,
	 and accelerates the particle in the vector direction.
	The amount of acceleration applied is directly proportional to magnitude.
	As with most acceleration actions, the amount of acceleration falls
	 off inversely with r2.
	But when r is small, the acceleration would be infinite, so epsilon
	 is always added to r.
	maxRadius defines the sphere of influence of this action.
	No particle further than maxRadius from the center is affected."

	<api: '?pOrbitPoint@@YAXMMMMMM@Z'
		ulong ulong ulong
		ulong ulong ulong
		none>
	^self invalidArgument!
   
pRandomAccel: dtype
	"Accelerate particles in random directions."

	^self pRandomAccel: dtype with: self defaults!
   
pRandomAccel: dtype
	a0: a0 a1: a1 a2: a2
	a3: a3 a4: a4 a5: a5
	a6: a6 a7: a7 a8: a8
	"Accelerate particles in random directions.
	For each particle, chooses an acceleration vector from the specified
	 domain and adds it to the particle's velocity.
	Reducing the time step, dt, will make a higher probability of being
	 near the original velocity after unit time.
	Smaller dt approach a normal distribution of velocity vectors instead
	 of a square wave distribution.
	"
	<api: '?pRandomAccel@@YAXW4PDomainEnum@@MMMMMMMMM@Z'
		ulong
		ulong ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		none>
	^self invalidArgument!
 
pRandomAccel: dtype with: arguments
	"Accelerate particles in random directions."

	| parameters |
	parameters := ReadStream on: (self cppFloats: arguments).
	^self pRandomAccel: dtype
		a0: parameters next a1: parameters next a2: parameters next
		a3: parameters next a4: parameters next a5: parameters next
		a6: parameters next a7: parameters next a8: parameters next!
 
pRandomDisplace: dtype
	"Immediately replace position with a position from the domain."

	^self pRandomDisplace: dtype with: self defaults!
  
pRandomDisplace: dtype with: arguments
	"Immediately replace position with a position from the domain."

	| parameters |
	parameters := ReadStream on: (self cppFloats: arguments).
	^self pRandomDisplace: dtype
		a0: parameters next a1: parameters next a2: parameters next
		a3: parameters next a4: parameters next a5: parameters next
		a6: parameters next a7: parameters next a8: parameters next!

pRandomVelocity: dtype
	"Immediately replace velocity with a velocity from the domain."

	^self pRandomVelocity: dtype with: self defaults!
  
pRandomVelocity: dtype with: arguments
	"Immediately replace velocity with a velocity from the domain."

	| parameters |
	parameters := ReadStream on: (self cppFloats: arguments).
	^self pRandomVelocity: dtype
		a0: parameters next a1: parameters next a2: parameters next
		a3: parameters next a4: parameters next a5: parameters next
		a6: parameters next a7: parameters next a8: parameters next!

pSink: killInside type: ptype
	"Kill particles with positions on wrong side of the zero domain."

	^self pSink: killInside type: ptype with: self defaults!
  
pSink: killInside type: ptype
	a0: a0 a1: a1 a2: a2
	a3: a3 a4: a4 a5: a5
	a6: a6 a7: a7 a8: a8
	"Kill particles with positions on wrong side of the specified domain.
	If killInside is true, deletes all particles inside the given domain.
	If killInside is false, deletes all particles outside the given domain.
	"
	<api: '?pSink@@YAX_NW4PDomainEnum@@MMMMMMMMM@Z'
		boolean ulong
		ulong ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		none>
	^self invalidArgument!

pSink: killInside type: ptype with: arguments
	"Kill particles with positions on wrong side of the zero domain."

	| parameters |
	parameters := ReadStream on: (self cppFloats: arguments).
	^self pSink: killInside type: ptype
		a0: parameters next a1: parameters next a2: parameters next
		a3: parameters next a4: parameters next a5: parameters next
		a6: parameters next a7: parameters next a8: parameters next!

pSinkVelocity: killInside type: dType
	"Kill particles with velocities on wrong side of the zero domain."

	^self pSinkVelocity: killInside type: dType with: self defaults!
 
pSinkVelocity: killInside type: dType with: arguments
	"Kill particles with velocities on wrong side of the zero domain."

	| parameters |
	parameters := ReadStream on: (self cppFloats: arguments).
	^self pSinkVelocity: killInside type: dType
		a0: parameters next a1: parameters next a2: parameters next
		a3: parameters next a4: parameters next a5: parameters next
		a6: parameters next a7: parameters next a8: parameters next!
   
pSize: x
	"Set a single particle size."

	^self
		pSize: x
		y: (self cppFloat: 1) asParameter
		z: (self cppFloat: 1) asParameter!
   
pSize: x y: y z: z
	"Set a single particle size."

	<api: '?pSize@@YAXMMM@Z'
		ulong ulong ulong
		none>
	^self invalidArgument!
  
pSizeD: dtype
	"Specify the size domain of new particles.
	All new particles will have a size chosen randomly from within the specified domain.
	NOTES: The size does not affect any particle dynamics, including acceleration and bouncing.
	 It is merely a triple of rendering attributes, like color, and can be interpreted at the whim
	 of the application programmer (thats you). In particular, the three components do not
	 need to be used together as three dimensions of the particles size. For example, one
	 could be interpreted as radius, another as length, and another as density.
	"

	^self pSizeD: dtype with: self defaults!
  
pSizeD: dtype with: arguments
	"Specify the size domain of new particles.
	All new particles will have a size chosen randomly from within the specified domain.
	NOTES: The size does not affect any particle dynamics, including acceleration and bouncing.
	 It is merely a triple of rendering attributes, like color, and can be interpreted at the whim
	 of the application programmer (thats you). In particular, the three components do not
	 need to be used together as three dimensions of the particles size. For example, one
	 could be interpreted as radius, another as length, and another as density.
	"

	| parameters |
	parameters := ReadStream on: (self cppFloats: arguments).
	^self pSizeD: dtype
		a0: parameters next a1: parameters next a2: parameters next
		a3: parameters next a4: parameters next a5: parameters next
		a6: parameters next a7: parameters next a8: parameters next!

pSource: particleRate type: dType
	"Add particles in the specified domain."

	^self pSource: particleRate type: dType with: self defaults!
   
pSource: particleRate type: dType
	a0: a0 a1: a1 a2: a2
	a3: a3 a4: a4 a5: a5
	a6: a6 a7: a7 a8: a8
	"Add particles in the specified domain.
	Adds new particles to the current particle group.
	The particle positions are chosen from the given domain.
	The particle colors, sizes, initial ages, velocities, and
	 secondary positions are chosen according to their
	 current domains.
	See pColor, pColorD, pSize, pStartingAge, pVelocity,
	 pVelocityD, pVertexB, pVertexBD, and pVertexBTracks.
	When pSource is called within an action list, the particle
	 attribute domains used are those that were current
	 when the pSource command was called within the
	 pNewActionList / pEndActionList block instead of when
	 pCallActionList is called.
	Note that this is unlike OpenGL.
	particleRate is the number of particles to add per unit time.
	If particleRate / dt is a fraction then pSource adjusts the
	 number of particles to add during this time step so
	 that the average number added per unit time is particleRate.
	"
	<api: '?pSource@@YAXMW4PDomainEnum@@MMMMMMMMM@Z'
		ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		none>
	^self invalidArgument!
  
pSource: particleRate type: dType with: arguments
	"Add particles in the specified domain."

	| parameters |
	parameters := ReadStream on: (self cppFloats: arguments).
	^self pSource: particleRate type: dType
		a0: parameters next a1: parameters next a2: parameters next
		a3: parameters next a4: parameters next a5: parameters next
		a6: parameters next a7: parameters next a8: parameters next!
 
pSpeedLimit: minSpeed
	"Clamp each particle's speed to the given min and no max."

	^self
		pSpeedLimit: minSpeed
		max: self pMaxFloat asParameter!
   
pStartingAge: age
	"Specify the initial age of new particles.
	The value age can be positive, zero, or negative.
	Giving particles different starting ages allows pKillOld to distinguish
	 between which to kill in interesting ways.
	"
	^self pStartingAge: age stdev: self zero asParameter!
  
pStartingAge: age stdev: stdev
	"Specify the initial age of new particles.
	The value age can be positive, zero, or negative.
	Giving particles different starting ages allows pKillOld to distinguish
	 between which to kill in interesting ways.
	Setting stdev to a non-zero value will give the particles an initial age
	 with a normal distribution with mean age and standard deviation stdev.
	When many particles are created at once this allows a few particles
	 to die at each time step, yielding a more natural effect.
	"
	<api: '?pStartingAge@@YAXMM@Z'
		ulong ulong
		none>
	^self invalidArgument!
   
pTargetSize: x y: y z: z
	"Change sizes of all particles toward the specified size."

	| zero |
	zero := self zero asParameter.
	^self
		pTargetSize: x y: y z: z
		scaleX: zero scaleY: zero scaleZ: zero!
  
pVelocity: x y: y z: z
	"Specify the initial velocity of new particles.
	This is shorthand for #pVelocityD:...
	"
	<api: '?pVelocity@@YAXMMM@Z'
		ulong ulong ulong
		none>
	^self invalidArgument!
  
pVelocityD: dtype
	"Specify the initial velocity domain of new particles."

	^self pVelocityD: dtype with: self defaults!

pVelocityD: dtype
	a0: a0 a1: a1 a2: a2
	a3: a3 a4: a4 a5: a5
	a6: a6 a7: a7 a8: a8
	"Specify the initial velocity domain of new particles.	"
	<api: '?pVelocityD@@YAXW4PDomainEnum@@MMMMMMMMM@Z'
		ulong
		ulong ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		none>
	^self invalidArgument!

pVelocityD: dtype with: arguments
	"Specify the initial velocity domain of new particles."

	| parameters |
	parameters := ReadStream on: (self cppFloats: arguments).
	^self pVelocityD: dtype
		a0: parameters next a1: parameters next a2: parameters next
		a3: parameters next a4: parameters next a5: parameters next
		a6: parameters next a7: parameters next a8: parameters next!
  
pVertex: x y: y z: z
	"Add a single particle at the specified location."

	<api: '?pVertex@@YAXMMM@Z'
		ulong ulong ulong
		none>
	^self invalidArgument!
 
pVertexBTracks
	"Specify how the secondary position of each new particle is chosen."

	^self pVertexBTracks: true!
   
pVortex: x y: y z: z
	axisX: axisX axisY: axisY axisZ: axisZ
	"Swirl particles around a vortex.
	x@y@z and axis define an infinite line, where center represents
	 the tip of the vortex and axis is a vector along the line,
	 the length of which is irrelevant.
	"
	^self
		pVortex: x y: y z: z
		axisX: axisX axisY: axisY axisZ: axisZ
		magnitude: 1 asParameter
		epsilon: self pEPS asParameter
		maxRadius: self pMaxFloat asParameter!
   
pVortex: x y: y z: z
	axisX: axisX axisY: axisY axisZ: axisZ
	magnitude: magnitude
	"Swirl particles around a vortex.
	x@y@z and axis define an infinite line, where center represents
	 the tip of the vortex and axis is a vector along the line,
	 the length of which is irrelevant.
	"
	^self
		pVortex: x y: y z: z
		axisX: axisX axisY: axisY axisZ: axisZ
		magnitude: magnitude
		epsilon: self pEPS asParameter
		maxRadius: self pMaxFloat asParameter!

pVortex: x y: y z: z
	axisX: axisX axisY: axisY axisZ: axisZ
	magnitude: magnitude
	epsilon: epsilon
	"Swirl particles around a vortex.
	x@y@z and axis define an infinite line, where center represents
	 the tip of the vortex and axis is a vector along the line,
	 the length of which is irrelevant.
	"
	^self
		pVortex: x y: y z: z
		axisX: axisX axisY: axisY axisZ: axisZ
		magnitude: magnitude
		epsilon: epsilon
		maxRadius: self pMaxFloat asParameter!
   
pVortex: x y: y z: z
	axisX: axisX axisY: axisY axisZ: axisZ
	magnitude: magnitude
	epsilon: epsilon
	maxRadius: maxRadius
	"Swirl particles around a vortex.
	x@y@z and axis define an infinite line, where center represents
	 the tip of the vortex and axis is a vector along the line,
	 the length of which is irrelevant.
	As with most acceleration actions, the amount of acceleration falls
	 off inversely with r2 to the center.
	But when r is small, the acceleration would be infinite,
	 so epsilon is always added to r.
	Using an epsilon similar in magnitude to magnitude can increase
	 the range of influence of the vortex. max_radius defines the
	 sphere of influence of this action.
	No particle further than max_radius from the vortex center is affected.
	NOTES: pVortex immediately displaces particle positions, unlike
	 most actions which affect particle velocities, so unsatisfying
	 results may occur when used with the pAvoid or pBounce actions.
	In particular, particles may be displaced to the opposite side
	 of the surface without bouncing off it.
	pVortex currently does not pull the particles up or down along
	 the axis like a tornado.
	This will be saved for a future release.
	If you can suggest an implementation, feel free to send it to me.
	Particles can be attracted toward the axis of the vortex using #pOrbitLine:...
	"
	<api: '?pVortex@@YAXMMMMMMMMM@Z'
		ulong ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		none>
	^self invalidArgument!

UnderTestpAvoid: magnitude epsilon: epsilon
	lookAhead: lookAhead dtype: dtype
	a0: a0 a1: a1 a2: a2
	a3: a3 a4: a4 a5: a5
	a6: a6 a7: a7 a8: a8
	"Steer particles away from a domain of space.
	Particles are tested to see whether they will pass from being outside
	 the specified domain to being inside it within lookAhead time units
	 from now if the next #pMove:... action were to occur now.
	magnitude tells how drastically the particle velocities are modified
	 to avoid the obstacle at each time step.
	As with most acceleration actions, the amount of acceleration falls
	 off inversely with r2. But when r is small, the acceleration would
	 be infinite, so epsilon is always added to r.
	The specific direction and amount of turn is dependent on the kind
	 of domain being avoided.
	NOTES: At present the only domains for which pAvoid is implemented
	 are PDSphere, PDRectangle, PDTriangle, PDDisc and PDPlane.
	"
	<api: '?pAvoid@@YAXMMMW4PDomainEnum@@MMMMMMMMM@Z'
		ulong ulong ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		none>
	^self invalidArgument!

UnderTestpCallActionList: actionListNum
	"Apply the specified action list to the current particle group."

	<api: '?pCallActionList@@YAXH@Z' long none>
	^self invalidArgument!
 
UnderTestpCopyGroup: groupNum index: index count: count
	"Copy particles from the specified group into the current group.
	Copy particles from the specified particle group, groupNum,
	 to the current particle group. Only 'count' particles, starting
	 with number index are copied. Of course, the number of particles
	 actually copied is bounded by the available space in the
	 current particle group, and the number of particles actually
	 in the source particle group. The particles are added in order
	 to the end of the current group.
	index is the index of the first particle in the source particle
	 group to be copied."

	<api: '?pCopyGroup@@YAXHHH@Z' long long long none>
	^self invalidArgument!
 
UnderTestpDeleteActionLists: actionListNum count: actionListCount
	"Delete one or more consecutive action lists.
	Deletes actionListCount action lists, with actionListNum
	 being the list number of the first one.
	The lists must be numbered sequentially, and must all exist.
	This removes the specified action lists from existence."

	<api: '?pDeleteActionLists@@YAXHH@Z'
		long long
		none>
	^self invalidArgument!
   
UnderTestpEndActionList
	"End the creation of a new action list."

	<api: '?pEndActionList@@YAXXZ' none>
	^self invalidArgument!

UnderTestpGenActionLists: count
	"Generate a block of empty action lists.
	All list numbers are in sequential order starting with the first list.
	Returns the action list number of the first allocated list.
	Valid action list numbers are non-negative."

	<api: '?pGenActionLists@@YAHH@Z'
		long
		ulongReturn>
	^self invalidArgument!
  
UnderTestpGetGroupCount
	"Returns the number of particles existing in the current group."

	<api: '?pGetGroupCount@@YAHXZ' ulongReturn>
	^self invalidArgument!
 
UnderTestpGetParticles: index0
	count: count
	position: positionHolder
	color: colorHolder
	velocity: velocityHolder
	size: sizeHolder
	age: ageHolder
	"Copy particles from the current group to application memory.
	Copies at most count particles beginning with the index-th (-1)
	 particle in the current particle group into memory already
	 allocated by the application. Three floats are returned for
	 the position of each particle, representing its x,y,z location.
	Four floats are returned for the color of each particle, representing
	 its R,G,B,A color. Three floats are returned for the velocity
	 of each particle, representing its dx,dy,dz direction vector.
	Three floats are returned for the size of each particle, representing
	 whatever the application wants them to. One float is returned for
	 the age of each particle.
	Returns the number of particles copied to application memory.
	Of course, the number of particles actually returned is bounded
	 by count and by the number of particles actually in the particle
	 group minus index.
	If verts, color, vel, size or age is 'nil' then no position, color,
	 velocity, size or age data, respectively, will be returned.
	index and count must be at least 0 and less than the number
	 of particles. index + count must be less than the number
	 of particles.
	WARNING: As with all arrays in C, the index of the first particle
	 is zero."

	<api: '?pGetParticles@@YAHHHPAM0000@Z'
		long long
		struct struct struct struct struct
		ulongReturn>
	^self invalidArgument!
  
UnderTestpGravitate: magnitude epsilon: epsilon maxRadius: maxRadius
	"Accelerate each particle toward each other particle.
	Each particle is accelerated toward each other particle in
	 the group by an amount proportional to magnitude.
	As with most acceleration actions, the amount of acceleration
	 falls off inversely with r2. But when r is small, the acceleration
	 would be infinite, so epsilon is always added to r.
	maxRadius defines the sphere of influence of this action.
	No particle further than maxRadius from another particle is affected.
	NOTES: This action is more computationally intensive than the
	 others are because each particle is affected by each other particle.
	"
	<api: '?pGravitate@@YAXMMM@Z'
		ulong ulong ulong
		none>
	^self invalidArgument!
  
UnderTestpMatchVelocity: magnitude epsilon: epsilon maxRadius: maxRadius
	"Modify each particles velocity to be similar to that of its neighbors.
	Each particle is accelerated toward the weighted mean of the velocities
	 of other particles in the group by an amount proportional to magnitude.
	As with most acceleration actions, the amount of acceleration falls
	 off inversely with r2 to the other particles.
	But when r is small, the acceleration would be infinite, so epsilon
	 is always added to r.
	Using an epsilon similar in magnitude to magnitude can increase the
	 range of influence of nearby particles on this particle.
	NOTES: This action is more computationally intensive than the others
	 are because each particle is affected by each other particle.
	"
	<api: '?pMatchVelocity@@YAXMMM@Z'
		ulong ulong ulong
		none>
	^self invalidArgument!
 
UnderTestpNewActionList: actionListNum
	"Begin the creation of the specified action list.
	The actionListNum must have already been generated using #pGenActionLists:...
	Most calls other than actions cannot be made between a call to #pNewActionList:...
	 and the corresponding call to #pEndActionList."

	<api: '?pNewActionList@@YAXH@Z' long none>
	^self invalidArgument!
  
UnderTestpRandomDisplace: dtype
	a0: a0 a1: a1 a2: a2
	a3: a3 a4: a4 a5: a5
	a6: a6 a7: a7 a8: a8
	"Immediately replace position with a position from the domain.
	For each particle, chooses a displacement vector from the
	 specified domain and adds it to the particle's position.
	Reducing the time step, dt, will make a higher probability
	 of being near the original position after unit time.
	Smaller dt approach a normal distribution of particle positions
	 instead of a square wave distribution.
	"
	<api: '?pRandomDisplace@@YAXW4PDomainEnum@@MMMMMMMMM@Z'
		ulong
		ulong ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		none>
	^self invalidArgument!
  
UnderTestpRandomVelocity: dtype
	a0: a0 a1: a1 a2: a2
	a3: a3 a4: a4 a5: a5
	a6: a6 a7: a7 a8: a8
	"Immediately replace velocity with a velocity from the domain.
	For each particle, sets the particle's velocity vector to a random
	 vector in the specified domain.
	This function is not affected by dt.
	If you can think of an appropriate way for this to vary with dt, let me know.
	"
	<api: '?pRandomVelocity@@YAXW4PDomainEnum@@MMMMMMMMM@Z'
		ulong
		ulong ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		none>
	^self invalidArgument!

UnderTestpRestore: timeLeft
	"Over time, restore particles to their secondary positions.
	Computes a new velocity for each particle that will make the
	 particle arrive at its positionB at the specified amount of time
	 in the future.
	The curved path that the particles take is a parametric quadratic.
	Once the specified amount of time has passed, the particles
	 are clamped to their positionB and their velocities are set to 0
	 to freeze them in place.
	If #pRestore:  is sent in immediate mode, it is the application's
	 responsibility to decrease timeLeft by dt on each call.
	When in an action list, timeLeft gets decremented automatically.
	The positionB attribute of each particle is typically the particle's
	 position when it was created, or it can be specified within a
	 domain.
	This is controlled by pVertexBTracks, pVertexB, and pVertexBD.
	The positionB can be set at any time to the particle's current
	 position using the pCopyVertexB action.
	"
	<api: '?pRestore@@YAXM@Z' ulong none>
	^self invalidArgument!

UnderTestpSetMaxParticles: count
	"Change the maximum number of particles in the current group.
	If necessary, this will delete particles from the end of the particle
	 group, but no other particles will be deleted."

	<api: '?pSetMaxParticles@@YAHH@Z'
		long
		ulongReturn>
	^self invalidArgument!
  
UnderTestpSinkVelocity: killInside type: dType
	a0: a0 a1: a1 a2: a2
	a3: a3 a4: a4 a5: a5
	a6: a6 a7: a7 a8: a8
	"Kill particles with velocities on wrong side of the specified domain.
	If killInside is true, deletes all particles whose velocity vectors
	 are inside the given domain.
	If killInside is false, deletes all particles whose velocity vectors
	 are outside the given domain.
	This allows particles to die when they turn around, get too fast
	 or too slow, etc.
	"
	<api: '?pSinkVelocity@@YAX_NW4PDomainEnum@@MMMMMMMMM@Z'
		boolean ulong
		ulong ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		none>
	^self invalidArgument!
  
UnderTestpSizeD: dtype
	a0: a0 a1: a1 a2: a2
	a3: a3 a4: a4 a5: a5
	a6: a6 a7: a7 a8: a8
	"Specify the size domain of new particles.
	All new particles will have a size chosen randomly from within the specified domain.
	The size values may be negative.
	NOTES: The size does not affect any particle dynamics, including acceleration and bouncing.
	 It is merely a triple of rendering attributes, like color, and can be interpreted at the whim
	 of the application programmer (thats you). In particular, the three components do not
	 need to be used together as three dimensions of the particles size. For example, one
	 could be interpreted as radius, another as length, and another as density.
	"
	<api: '?pSizeD@@YAXW4PDomainEnum@@MMMMMMMMM@Z'
		ulong
		ulong ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		none>
	^self invalidArgument!
   
UnderTestpSpeedLimit: minSpeed max: maxSpeed
	"Clamp each particle's speed to the given min and max.
	Computes each particle's speed (the magnitude of its
	 velocity vector) and if it is less than minSpeed or
	 greater than maxSpeed it is clamped to those bounds,
	 while preserving the velocity vector's direction.
	"

	<api: '?pSpeedLimit@@YAXMM@Z'
		ulong ulong
		none>
	^self invalidArgument!
 
UnderTestpTargetColor: x y: y z: z alpha: alpha scale: scale
	"Change color of all particles toward the specified color.
	Modifies the color and alpha of each particle to be scale percent
	 of the way closer to the specified color and alpha.
	scale is multiplied by dt before scaling the sizes.
	Thus, using smaller dt causes a slightly faster approach to the
	 target color.
	NOTES: This action makes all colors tend toward the specified,
	 uniform color. Future versions will have more actions that modify
	 color. Please send me suggestions (perhaps with
	 sample implementations).
	The value of scale will usually be very small (less than 0.01)
	 to yield a gradual transition."

	<api: '?pTargetColor@@YAXMMMMM@Z'
		ulong ulong ulong
		ulong ulong
		none>
	^self invalidArgument!
  
UnderTestpTargetSize: x y: y z: z
	scaleX: scaleX scaleY: scaleY scaleZ: scaleZ
	"Change sizes of all particles toward the specified size.
	Modifies the size of each particle to be scale percent of the
	 way closer to the specified size triple.
	This makes sizes grow asymptotically closer to the given size.
	scale is multiplied by dt before scaling the sizes.
	Thus, using smaller dt causes a slightly faster approach
	 to the target size.
	The separate scales for each component allow only selected
	 components to be scaled.
	"

	<api: '?pTargetSize@@YAXMMMMMM@Z'
		ulong ulong ulong
		ulong ulong ulong
		none>
	^self invalidArgument!

UnderTestpTargetVelocity: x y: y z: z scale: scale
	"Change velocity of all particles toward the specified velocity.
	Modifies the velocity of each particle to be scale percent
	 of the way closer to the specified velocity triple.
	This makes velocities grow asymptotically closer to the given velocity."

	<api: '?pTargetVelocity@@YAXMMMM@Z'
		ulong ulong ulong
		ulong none>
	^self invalidArgument!
   
UnderTestpTimeStep: dt
	"Specify the time step length.
	The Particle System API uses a discrete time approximation to all actions.
	This means that actions are applied to the particles at a particular instant in time
	 as if the actions effect accumulated over a small time interval, dt, with the world
	 being constant over the interval.
	The clock is then 'ticked' by the length of the interval and the actions can then
	 be reapplied with the particles having their updated values.
	This is the standard method of doing almost all time-varying simulations in
	 computer science.
	How does the time step, dt, relate to the application's frame rate?
	The easiest method is to apply the actions once per frame.
	If the application prefers to keep time in terms of seconds, dt can be set
	 to (1 / frames_per_second). But more often, it is easier for a time unit to
	 be one frame instead of one second. In this case, dt should be 1.0,
	 which is the default.
	For higher quality, the application can apply particle actions more than once
	 per frame. This provides smoother, more realistic results in many subtle ways.
	Suppose the application wanted to compute three animation steps for each
	 rendered frame. Set dt to 1/3 its previous value using pTimeStep, then loop
	 three times over all the action code that gets executed per frame, including
	 the calls to pMove. If using action lists, this can be simply a loop over the
	 pCallActionList call. The run-time results should be about the same, but
	 with fewer discrete approximation artifacts. Surprisingly enough, the actions
	 themselves usually take a small percentage of the CPU time, so using
	 a finer step size usually does not impact frame rate very much.
	In terms of numerical integration, particle actions can be thought of as
	 the first derivative of unknown functions dictating the particle attributes
	 (such as position) over time. In order to compute the particle attributes
	 these derivative functions must be integrated. Since closed form integration
	 doesn't make sense for most actions, Euler's method is used instead.
	 Euler's method is simply the method just described  the evaluation of the
	 derivative functions at a particular time and then incrementing the current
	 particle values by these derivative results times dt. In Euler's method,
	 the smaller the dt, the more accurate the results.
	NOTES: Unlike with other state setting calls, action lists execute using
	 the current dt value set by pTimeStep, rather than the time step value
	 that was current when the action list was created. Making action lists
	 independent of time step size allows the time step to be changed without
	 recompiling the action list.
	"
	<api: '?pTimeStep@@YAXM@Z' ulong none>
	^self invalidArgument!
   
UnderTestpVertexB: x y: y z: z
	"Specify the secondary position of new particles.
	This is shorthand for #pVertexBD:..."

	<api: '?pVertexB@@YAXMMM@Z'
		ulong ulong ulong
		none>
	^self invalidArgument!
   
UnderTestpVertexBD: dtype
	a0: a0 a1: a1 a2: a2
	a3: a3 a4: a4 a5: a5
	a6: a6 a7: a7 a8: a8
	"Specify the secondary position domain of new particles.
	Set the domain from which the positionB of each particle
	 will be randomly chosen when it is created.
	But if #pVertexBTracks is set to true (the default), then the
	 positionB will instead be the same as the initial particle
	 position."
	<api: '?pVertexBD@@YAXW4PDomainEnum@@MMMMMMMMM@Z'
		ulong
		ulong ulong ulong
		ulong ulong ulong
		ulong ulong ulong
		none>
	^self invalidArgument!
   
UnderTestpVertexBTracks: doCopy
	"Specify how the secondary position of each new particle is chosen.
	Specify how the positionB of each new particle emitted will be chosen.
	The positionB is the destination position or initial position of particles.
	See #pRestore for a use of positionB.
	If doCopy is true then when a particle is created its positionB will
	 be the same as its initial particle position (the default).
	If doCopy is false then when a particle is created its positionB will
	 be chosen from the VertexB domain.
	"
	<api: '?pVertexBTracks@@YAX_N@Z'
		boolean
		none>
	^self invalidArgument!

version
	"Returns the major and minor version number of this release of the API."

	^#(1 20)!

zero
	" Private - Returns 0.0 as a valid cpp float (to be passed as an API parameter). "

	^self cppFloat: 0! !
